/*    Clock Firmware for the ESP32 series of boards and nixie clock PN Z57xM6DV2 available at www.neonixie.com/Z57xM6DV2/
 *    Copyright © 2023 Neonixie.com, All Spectrum
 *    Permission for use and modification is provided for clocks purchased from us.
 *    Software is provided as-is, with no warranty expressed or implied.
 *    
 *    YOU SHOULD NOT PUT THIS DEVICE ON THE PUBLIC INTERNET - KEEP IT BEHIND A ROUTER/FIREWALL!
 *    
 *    Auth info
 *    - Web based firmware update:
 *            Z57XM6DV2:  http username = "z57clock"   password = "neonixie.com";
 *            IV-9-6D-RR: http username = "iv9clock"   password = "neonixie.com";
 *    - Arduino OTA firmware update, username = "neonixie.com"
 *    - mDNS hostname, usually used with Arduino OTA or local network:   
 *            Z57XM6DV2:  Z57clock-MAC_ADDRESS (Z57clock-MAC_ADDRESS.local)
 *            IV-9-6D-RR: iv9clock-MAC_ADDRESS (iv9clock-MAC_ADDRESS.local)
 *    - SSID when clock is in AP mode/SETUP mode (or when unable to connect to configured access point):
 *            Z57XM6DV2:  SSID = "z57clock"   password = "neonixie.com"
 *            IV-9-6D-RR: SSID = "iv9clock"   password = "neonixie.com"
 *            
 *    Revision History
 *    0.90  07-31-2022  First public release
 *    
 *    0.91  not final   NOTE: Version 0.91 BETA4 is considered release quality since no major bugs have been found   
 *                      Add Cathode poisoning prevention options
 *                      Add date display options
 *                      Add leading zero blanking option
 *                      Add AutoConnect OTA Web based uploading of new firmware
 *                      Add temperature and humidity compensation options
 *                      Add temperature display every X seconds and duration options, removed temperature display tickbox
 *                      Add correct temperature display code for degree F at or above 100
 *                      Moved struct tm timeinfo; to GLOBAL
 *                      Add total uptime in days and total cumulative runtime in days (stored in non-volatile flash)
 *                      Add temperature, humidity, lux, version, uptime and total runtime to main web page on device
 *                      Code cleanup, move original time code (before timezone support) to extra-code.txt
 *                      Changed bootup IP address display to full IP address (prior code only displayed last two octets)
 *                      Added code to reset permanent (flash / like eeprom) variables in case of corruption or incorrect values cause a crash that we cannot recover from
 *                        This is done by grounding pin 15 on the ESP while the board is reset/powered up. The display will flash all 0's three times to indicate 
 *                        that flash variables have been reset to default values. TOTAL_RUNTIME is preserved.
 *                      Added watchdog timer of 300 seconds. Mainly to force the clock to attempt to reconnect to wifi when in soft AP mode. Should not get triggered in any other mode.
 *                      Added config.autoReconnect = true; to make wifi attempt a reconnect to known routers. Without this if the wifi router went offline, the clock would not reconnect.
 *                      BETA4, Removed a return from initTime, was causing timezone not to set sometimes at initial bootup
 *                      BETA4, Removed free memory display on main http page
 *                      BETA4, Changed light sensor default to off in eeprom
 *                      BETA4, Added default brightness variable in setup(), without this if light sensor was selected and none was present display would be blank
 *                      BETA4, Change default sleep time to display IP address from 2s to 4s
 *                      BETA4, Removed extra getLocalTime function and a few lines around it in cathodepp() function
 *                      BETA4, Modified ArduinoOTA.setHostname to shorten local hostname, was full MAC now Z57clock-[last-two-octets-of-MAC]
 *                      
 *    0.92 BETA1 UNIV   FIRST UNIVERSAL CODEBASE FOR IV-9-6D-RR and Z57XM6DV2. Sourced from Z57XM6DV2 code v0.91 BETA4
 *                      Arduino IDE 1.8.19, ESP32 board manager v2.0.6, AutoConnect v1.4.0, Adafruit_BMP280_Library v2.6.6, Adafruit_AHTX0 v2.0.1
 *                      MAIN CHANGES:
 *                      Added preprocessor defines for clock type and autodetection of board type used
 *                      Several serial_debugxxx preprocessor defines added for Serial.Printxxx statements to remove serial output when not needed
 *                      Driver mapping include file name changed to driver-mapping-definitions.h
 *                      Added full alphabet support to driver-mapping-definitions.h for IV-9 clock
 *                      HTML config page text moved to html-definitions.h
 *                      Added esp_partition.h and related code to display partition information, mainly sizes, linked at main http page
 *                      Added code to read BMP280, added read options on configuration page and display results on main web page
 *                      Added code to enable/disable colon indicators to main and config web pages
 *                      Added code to handle "Tubes on bottom of PCB" to main and config web pages for IV-9 clock
 *                      Added code for "Alternate Display" option. Options are standard (clock), altimeter, RSSI, Lux and Barometer (HG)
 *                      Moved decimal point code outside of write_display function and related changes
 *                      PWM dimming set to 1,600 hz
 *                      Added code to set default brightness in settings to 165 for IV-9-6D-RR and 255 for Z57XM6DV2
 *                      Removed delay(20) in main loop
 *                      Changes made to various hostname settings to fix issues caused by upgrading the ESP32 board manager
 *                        When upgrading the ESP32 board manager from 1.0.6 to 2.0.5 hostnames (softAP hostname, wifi/dhcp hostname and MDNS) were initially not setting correctly, they would be ESP32xxxx defaults
 *                        now working as before after several code modifications
 *                      
 *    0.92 BETA2 UNIV   UNIVERSAL CODEBASE FOR IV-9-6D-RR and Z57XM6DV2
 *                      Arduino IDE 1.8.19, ESP32 board manager v2.0.8, AutoConnect v1.4.0, Adafruit_BMP280_Library v2.6.6, Adafruit_AHTX0 v2.0.1
 *                      PubSubClient by Nick O’Leary v2.8.0, AnyRtttl v2.2.1
 *                      MAIN CHANGES:
 *                      - Added digit cross fading
 *                      - Added MQTT client to publish sensor values and set brightness of clock
 *                      - Added alarm feature, including audio playback of RTTTL ringtones via a piezo element
 *                      - Added timed brightness feature to automatically adjust clock brightness on specific days/times
 *                      - Added feature on IV-9 clock to keep unlit segments warm to lower inrush current and hopefully make the tubes last longer (Segment keep-warm ratio in Advanced Config)
 *                      - Added field to allow a user specified NTP server
 *                      - Added Quickboot option that will bypass Lamp Check and IP display when enabled
 *                      - Added "Clock Config Advanced" and moved less used options to that page
 *                      - Added minimum brightness setting (for sensor controlled brightness) in advanced config. 
 *                        Changed Z57 mid level lux controlled brightness to a value between max bright and min bright. This now matches how the IV9 clock works.
 *                      - Modified brightness code to ramp up display on bootup to prevent spike LOADS from resetting ESP32
 *                      - Modified main brightness code to ramp up and down all display brightness requests gradually, this will help with LOAD regulation especially on IV-9 clock
 *                      - Modified how the display is updated, moved away from directly calling WRITE_DISPLAY to updating digit variables directly
 *                        mainly to account for digit cross fading, added hardware timer at 1600hz, etc..
 *                      - Upgraded ESP32 board manager from v2.0.6 to v2.0.8. Mainly to fix incorrect maximum sketch size bug  https://github.com/espressif/arduino-esp32/issues/8026
 *                      
 */
// WIFI CONFIG / CONNECTION / WEB SERVER BASED on https://hieromon.github.io/AutoConnect/
// NTP BASED on https://RandomNerdTutorials.com/esp8266-nodemcu-date-time-ntp-client-server-arduino/
// EEPROM / "PREFERENCES" https://randomnerdtutorials.com/esp32-save-data-permanently-preferences/

//  THIS IS WHAT IS NORMALLY USED FOR DEV using original esp32-devkitc-ve BOARD, Also works with ESP32-DevKitC-32D
// Board Node32s    or    DOIT ESP32 DEVKIT V1  seems to work too! // upload speed 921600 // flash freq 80mhz // part scheme default
// 01/13/2023 BOARD UPDATE esp32-devkitc-ve works well as "ESP32 Wrover Kit (all versions)" and has more board options to choose from

const char VERSION[] = "0.92 BETA2";
const char compile_date[] = __DATE__ " " __TIME__;

// TYPE OF CLOCK, ONLY UNCOMMENT ONE
#define Z57XM6DV2
//#define IV_9_6D_RR

#ifdef IV_9_6D_RR 
  #define CLOCKMODEL "IV-9-6D-RR"
  #define CLOCKTYPE "Numitron"
  #define WRITE_DISPLAY(a,b,c,d,e,f,g,h,i,j,k,l) WRITE_DISPLAY_IV9(a,b,c,d,e,f,g,h,i,j,k,l)
  #define DIGIT_NEGATIVE 36
  #define DIGIT_BLANK 37
  #ifdef ARDUINO_ADAFRUIT_QTPY_ESP32S3_NOPSRAM  // Works but not extensively tested, QIO 80mhz, Flash Size 8MB, Partition Default (3MB APP/1.5MB SPIFFS), Arduino and Events on Core 1
    #define CLOCK_PIN 18  // A0
    #define RCK_PIN 17    // A1
    #define OE_PIN 9      // A2
    #define DATA_PIN 8    // A3 
    #define EEPROM_RESET_PIN 16
    #define BUZZER_PIN 36   // TESTED
    #define ALARM_PIN 16   // NOT YET TESTED
  #endif
  #ifdef ARDUINO_ADAFRUIT_QTPY_ESP32C3          // Works but not extensively tested, QIO 80mhz, Flash Size 4MB, Minimal SPIFFS (1.9MB APP OTA / 190KB SPIFFS)
    #define CLOCK_PIN 4   // A0
    #define RCK_PIN 3     // A1
    #define OE_PIN 1      // A2
    #define DATA_PIN 0    // A3
    #define EEPROM_RESET_PIN 20
    #define BUZZER_PIN 10   // NOT YET TESTED
    #define ALARM_PIN 20   // NOT YET TESTED
  #endif
  #ifdef ARDUINO_XIAO_ESP32C3 // Flash size 4MB, Minimal SPIFFS (1.9MB APP OTA / 190KB SPIFFS), Flash FREQ 80mhz, Flash Mode QIO
    #define CLOCK_PIN 2   // A0
    #define RCK_PIN 3     // A1
    #define OE_PIN 4      // A2
    #define DATA_PIN 5    // A3
    #define EEPROM_RESET_PIN 20
    #define BUZZER_PIN 8   // TESTED
    #define ALARM_PIN 9   // TESTED on PIN 9 // NOTE: DID NOT WORK ON GPIO PIN 20.. which is also listed as the RX pin
  #endif  
  #ifdef ARDUINO_ESP32_WROVER_KIT // THIS IS "ESP32 Wrover Kit (all versions)" in the Arduino IDE, flash freq 80mhz, QIO, 8mb flash size, 8M with spiffs (3MB APP/1.5MB SPIFFS), PSRAM enabled
    #define CLOCK_PIN 19
    #define RCK_PIN 18
    #define OE_PIN 2
    #define DATA_PIN 4
    #define EEPROM_RESET_PIN 15
    #define BUZZER_PIN 32
    #define ALARM_PIN 33
  #endif
//  #ifdef ARDUINO_ESP32_DEV    // THIS IS "ESP32 Wrover Module" in the Arduino IDE, TEST ONLY, USE INSTEAD "ESP32 Wrover Kit (all versions)"
//    #define OE_PIN 2
//    #define DATA_PIN 4
//    #define CLOCK_PIN 19
//    #define RCK_PIN 18
//  #endif
#endif

#ifdef Z57XM6DV2    // Z57XM6DV2 CURRENTLY ONLY USES ONE BOARD, SO NO NEED TO CHECK BOARD TYPE 
  #define CLOCKMODEL "Z57XM6DV2"
  #define CLOCKTYPE "Nixie"
  #define WRITE_DISPLAY(a,b,c,d,e,f,g,h,i,j,k,l) WRITE_DISPLAY_Z57(a,b,c,d,e,f,g,h,i,j,k,l)
  #define DIGIT_NEGATIVE 10 // WE DON'T HAVE A NEGATIVE ON A NIXIE, NOT THIS ONE AT LEAST! SO WE JUST SET IT TO BLANK
  #define DIGIT_BLANK 10
  #define OE_PIN 19
  #define CLOCK_PIN 18
  #define DATA_PIN 4
  #define EEPROM_RESET_PIN 15
  #define BUZZER_PIN 32
  #define ALARM_PIN 33
#endif

// 1068881 with serial debug on
// 1067129 with serial debug off, size comparison, a difference of 1,752 bytes, tested with pre-release version
#define SERIAL_DEBUG 0

#if SERIAL_DEBUG == 1
  #define serial_debug(x) Serial.print(x)
  #define serial_debug_x2(x,y) Serial.print(x,y)
  #define serial_debugln(x) Serial.println(x)
  #define serial_debug_printf(x,y) Serial.printf(x,y)
#else
  #define serial_debug(x)
  #define serial_debug_x2(x,y)
  #define serial_debugln(x)
  #define serial_debug_printf(x,y)
#endif

#include <WiFi.h>
#include <NTPClient.h>    // I BELIEVE THIS IS https://github.com/arduino-libraries/NTPClient   NTPClient by Fabrice Weinberg v3.2.0
#include <WiFiUdp.h>
#include <Wire.h>
#include <BH1750.h>
#include <Preferences.h>    // FOR "EEPROM" SAVING, IT'S CALLED "Preferences" in Arduino
Preferences preferences;

#include "driver-mapping-definitions.h"    // Mapping for HV5122 shift registers.. SEC_ONE_MAP, SEC_TEN_MAP, etc... and IV-9 driver ICs
#include "html-definitions.h"    // Web page code

// OTA STUFF
#include <ArduinoOTA.h>
#include <ESPmDNS.h>

// FOR ESP PARTITION READING
#include "esp_partition.h"

// Watchdog timer
#include <esp_task_wdt.h>

// AHT20 CODE BASED ON https://github.com/adafruit/Adafruit_AHTX0
// INSTALL FROM LIBRARY MANAGER AND SEARCH FOR AHT20, install dependencies as well
#include <Adafruit_AHTX0.h>
Adafruit_AHTX0 aht;

// BMP280 CODE https://github.com/adafruit/Adafruit_BMP280_Library
// INSTALL IN LIBRARY MANAGER AND SEARCH FOR "Adafruit BMP280"
#define BMP_ENABLE
#ifdef BMP_ENABLE
  #include <Adafruit_BMP280.h>
  Adafruit_BMP280 bmp; // I2C
#endif
float bmp_PA;
float bmp_HG;
char bmp_HG_STRING_BUFFER[8];
float bmp_METERS;
float bmp_FEET;
float bmp_TEMPERATURE_C;

struct tm timeinfo;
char TIME_STRING[150];

float HUMIDITY;
float TEMPERATURE_C;
float TEMPERATURE_F;

float lux;

char tmp_hostname[30];

int SETTING_BRIGHTNESS;
int SETTING_MINBRIGHTNESS;
int SETTING_LUXMETER;
int SETTING_12H;
int SETTING_LZBLANK;
int SETTING_DATEFORMAT;
int SETTING_DATEDISP_SEC;
int SETTING_DATEDISP_DUR;
String SETTING_TIMEZONE;
//int SETTING_DISPLAYTEMP;
int SETTING_TEMP_IN_C;
int SETTING_TEMPDISP_SEC;
int SETTING_TEMPDISP_DUR;
int SETTING_TEMP_COMP;
int SETTING_HUMIDITY_COMP;
int SETTING_CATHODEPP;
int SETTING_CATHODEPPSTART;
int SETTING_CATHODEPPDUR;
int SETTING_CATHODEPPREV;
int SETTING_CATHODEPPREVD;

int SETTING_TUBEONBOTTOM;

int SETTING_COLON;

int SETTING_ALTDISPLAY;

int SETTING_BMP;

int SETTING_TEMPHUM;

String SETTING_NTPSERVER;

int SETTING_MQTTENABLE;
int SETTING_MQTTBRIGHT;
String SETTING_MQTTSERVER;
String SETTING_MQTTPORT;
String SETTING_MQTTUSERNAME;
String SETTING_MQTTPASSWORD;
String SETTING_MQTTROOT;

int SETTING_MQTTTEMP_F;
int SETTING_MQTTTEMP_C;
int SETTING_MQTTHUM;
int SETTING_MQTTLUX;
int SETTING_MQTTbmp_PA;
int SETTING_MQTTbmp_HG;
int SETTING_MQTTbmp_METERS;
int SETTING_MQTTbmp_FEET;
int SETTING_MQTTbmp_TEMP_C;
int SETTING_MQTTRSSI;

int SETTING_ALARM1ENABLE;
int SETTING_ALARM1SUN;
int SETTING_ALARM1MON;
int SETTING_ALARM1TUE;
int SETTING_ALARM1WED;
int SETTING_ALARM1THU;
int SETTING_ALARM1FRI;
int SETTING_ALARM1SAT;
int SETTING_ALARM1MONTH;
int SETTING_ALARM1DAY;
int SETTING_ALARM1HOUR;
int SETTING_ALARM1MIN;
int SETTING_ALARM1DUR;
int SETTING_ALARM1TUNE;
long ALARM1_DURATION = 88888;

int SETTING_ALARM2ENABLE;
int SETTING_ALARM2SUN;
int SETTING_ALARM2MON;
int SETTING_ALARM2TUE;
int SETTING_ALARM2WED;
int SETTING_ALARM2THU;
int SETTING_ALARM2FRI;
int SETTING_ALARM2SAT;
int SETTING_ALARM2MONTH;
int SETTING_ALARM2DAY;
int SETTING_ALARM2HOUR;
int SETTING_ALARM2MIN;
int SETTING_ALARM2DUR;
int SETTING_ALARM2TUNE;
long ALARM2_DURATION = 88888;

int SETTING_ALARM3ENABLE;
int SETTING_ALARM3SUN;
int SETTING_ALARM3MON;
int SETTING_ALARM3TUE;
int SETTING_ALARM3WED;
int SETTING_ALARM3THU;
int SETTING_ALARM3FRI;
int SETTING_ALARM3SAT;
int SETTING_ALARM3MONTH;
int SETTING_ALARM3DAY;
int SETTING_ALARM3HOUR;
int SETTING_ALARM3MIN;
int SETTING_ALARM3DUR;
int SETTING_ALARM3TUNE;
long ALARM3_DURATION = 88888;

long ALARM_ONETIMEPLAY_DURATION = 88888;

int ALARM_PREV_MIN;

String SETTING_RTTTL1, SETTING_RTTTL2, SETTING_RTTTL3;

int SETTING_TB1ENABLE;
int SETTING_TB1BRIGHT;
int SETTING_TB1SUN;
int SETTING_TB1MON;
int SETTING_TB1TUE;
int SETTING_TB1WED;
int SETTING_TB1THU;
int SETTING_TB1FRI;
int SETTING_TB1SAT;
int SETTING_TB1FROM;
int SETTING_TB1TO;

int SETTING_TB2ENABLE;
int SETTING_TB2BRIGHT;
int SETTING_TB2SUN;
int SETTING_TB2MON;
int SETTING_TB2TUE;
int SETTING_TB2WED;
int SETTING_TB2THU;
int SETTING_TB2FRI;
int SETTING_TB2SAT;
int SETTING_TB2FROM;
int SETTING_TB2TO;

int SETTING_TB3ENABLE;
int SETTING_TB3BRIGHT;
int SETTING_TB3SUN;
int SETTING_TB3MON;
int SETTING_TB3TUE;
int SETTING_TB3WED;
int SETTING_TB3THU;
int SETTING_TB3FRI;
int SETTING_TB3SAT;
int SETTING_TB3FROM;
int SETTING_TB3TO;

int SETTING_TB4ENABLE;
int SETTING_TB4BRIGHT;
int SETTING_TB4SUN;
int SETTING_TB4MON;
int SETTING_TB4TUE;
int SETTING_TB4WED;
int SETTING_TB4THU;
int SETTING_TB4FRI;
int SETTING_TB4SAT;
int SETTING_TB4FROM;
int SETTING_TB4TO;

int SETTING_KEEPWARMRATIO;

int SETTING_CROSSFADE;

int SETTING_QUICKBOOT;

int SETTING_ONETIMEPLAY;

int BRIGHTNESS;
int ACTUAL_BRIGHTNESS;

byte SEGMENT_KEEPALIVE_TIMER_RUNNING=0;

int DISPLAY_OTHER_THEN_TIME;

int makenoise = 0;

float TOTAL_RUNTIME;   // TOTAL TIME CLOCK HAS BEEN POWERED UP IN DAYS
int TOTAL_RUNTIME_SECONDS_COUNTER = 0;
float UPTIME = 0;       // UPTIME IN DAYS

BH1750 lightMeter;

// FOR TIMEZONE???  // FROM     https://randomnerdtutorials.com/esp32-ntp-timezones-daylight-saving/
#include "time.h"

// C:\Users\$YOUR-USERNAME\Documents\Arduino\libraries\AutoConnect\src\AutoConnectDefs.h
// Uncomment the following AC_DEBUG to enable debug output. IN THE ABOVE FILE!!!!!!!!
// #define AC_DEBUG ////////////// IN THE ABOVE FILE!!!!!!!!
// #define AUTOCONNECT_JSONDOCUMENT_SIZE   (16 * 1024)    // IMPORTANT!!!!! THIS DEFINE DOES NOT SEEM TO WORK.. MUST ADD THIS TO AutoConnectDefs.h  !!!!!!!!!!!!!! WE NEED THIS FOR THE ALARM WEB PAGE TO DISPLAY, SINCE IT'S LARGE AND RUNS OUT OF MEMORY OTHERWISE
#include <AutoConnect.h>  // CHECK http://IP/_ac  when connected!
#include <WebServer.h>
WebServer Server;
AutoConnect Portal(Server);
// ORIGINAL THAT DOES NOT WORK ANYMORE ON NEWER VERSION 12-25-2022, SEE PSK STUFF BELOW
// BUT MUST HAVE IT TO WORK?? something with config variable?
//AutoConnectConfig config("z57clock", "neonixie.com");    // This is the default SSID and PASSWORD used when the device can't connect and goes to "AP" mode
AutoConnectConfig config;

#include <PubSubClient.h>
WiFiClient wifiClient;
PubSubClient mqttClient(wifiClient);
char mqtt_string_buffer[20];

char mqtt_topic_temp_f[70];
char mqtt_topic_temp_c[70];
char mqtt_topic_hum[70];
char mqtt_topic_lux[70];
char mqtt_topic_bmp_PA[70];
char mqtt_topic_bmp_HG[70];
char mqtt_topic_bmp_METERS[70];
char mqtt_topic_bmp_FEET[70];
char mqtt_topic_bmp_TEMPERATURE_C[70];
char mqtt_topic_RSSI[70];

char mqtt_topic_subscribe_brightness[70];

boolean mqtt_reconnect() {
  mqttClient.setServer(SETTING_MQTTSERVER.c_str(), atoi(SETTING_MQTTPORT.c_str()));
  mqttClient.setCallback(mqttcallback);
  mqttClient.connect(tmp_hostname, SETTING_MQTTUSERNAME.c_str(), SETTING_MQTTPASSWORD.c_str());
  mqttClient.subscribe(mqtt_topic_subscribe_brightness);

  return mqttClient.connected();
}

int SECONDS_FRACTION_COUNTER=500;

// https://github.com/end2endzone/AnyRtttl  version 2.2.1
// IMPORTANT NOTE!!!!!!!! TO COMPILE ON THE WROOVER MODULE NEED TO CHANGE THE FOLLOWING IN src/anyrtttl.h
// BUG REPORT SUBMITTED https://github.com/end2endzone/AnyRtttl/issues/6
/*
    //typedef void (*DelayFuncPtr)(unsigned long);  // ORIGINAL, DOES NOT COMPILE WITH WROOVER ON LIBRARY v2.2.1
    typedef void (*DelayFuncPtr)(uint32_t);   // WORKS
*/
#include <anyrtttl.h>
#include <binrtttl.h>
#include <pitches.h>

//#define BUZZER_PIN 8 // ALREADY DEFINED
const char * SETTING_ALARMNOTE1 = "Dingdong:d=4,o=4,b=60:e,c";
const char * SETTING_ALARMNOTE2 = "Tetris:d=4,o=5,b=160:e6,8b,8c6,8d6,16e6,16d6,8c6,8b,a,8a,8c6,e6,8d6,8c6,b,8b,8c6,d6,e6,c6,a,2a,8p,d6,8f6,a6,8g6,8f6,e6,8e6,8c6,e6,8d6,8c6,b,8b,8c6,d6,e6,c6,a,a";
const char * SETTING_ALARMNOTE3 = "Mario:d=4,o=5,b=100:16e6,16e6,32p,8e6,16c6,8e6,8g6,8p,8g,8p,8c6,16p,8g,16p,8e,16p,8a,8b,16a#,8a,16g.,16e6,16g6,8a6,16f6,8g6,8e6,16c6,16d6,8b,16p,8c6,16p,8g,16p,8e,16p,8a,8b,16a#,8a,16g.,16e6,16g6,8a6,16f6,8g6,8e6,16c6,16d6,8b,8p,16g6,16f#6,16f6,16d#6,16p,16e6,16p,16g#,16a,16c6,16p,16a,16c6,16d6,8p,16g6,16f#6,16f6,16d#6,16p,16e6,16p,16c7,16p,16c7,16c7,p,16g6,16f#6,16f6,16d#6,16p,16e6,16p,16g#,16a,16c6,16p,16a,16c6,16d6,8p,16d#6,8p,16d6,8p,16c6";
const char * SETTING_ALARMNOTE4 = "Arkanoid:d=4,o=5,b=140:8g6,16p,16g.6,2a#6,32p,8a6,8g6,8f6,8a6,2g6";
const char * SETTING_ALARMNOTE5 = "StarWarsEnd:d=4,o=5,b=225:2c,1f,2g.,8g#,8a#,1g#,2c.,c,2f.,g,g#,c,8g#.,8c.,8c6,1a#.,2c,2f.,g,g#.,8f,c.6,8g#,1f6,2f,8g#.,8g.,8f,2c6,8c.6,8g#.,8f,2c,8c.,8c.,8c,2f,8f.,8f.,8f,2f";
const char * SETTING_ALARMNOTE6 = "Mozart:d=16,o=5,b=125:16d#,c#,c,c#,8e,8p,f#,e,d#,e,8g#,8p,a,g#,g,g#,d#6,c#6,c6,c#6,d#6,c#6,c6,c#6,4e6,8c#6,8e6,32b,32c#6,d#6,8c#6,8b,8c#6,32b,32c#6,d#6,8c#6,8b,8c#6,32b,32c#6,d#6,8c#6,8b,8a#,4g#,d#,32c#,c,c#,8e,8p,f#,e,d#,e,8g#,8p,a,g#,g,g#,d#6,c#6,c6,c#6,d#6,c#6,c6,c#6,4e6,8c#6,8e6,32b,32c#6,d#6,8c#6,8b,8c#6,32b,32c#6,d#6,8c#6,8b,8c#6,32b,32c#6,d#6,8c#6,8b,8a#,4g#";
const char * SETTING_ALARMNOTE7 = "HappyBirthday:o=5,d=4,b=125:8g.,16g,a,g,c6,2b,8g.,16g,a,g,d6,2c6,8g.,16g,g6,e6,c6,b,a,8f6.,16f6,e6,c6,d6,2c6,8g.,16g,a,g,c6,2b,8g.,16g,a,g,d6,2c6,8g.,16g,g6,e6,c6,b,a,8f6.,16f6,e6,c6,d6,2c6";

void rootPage() {
  char content[1200]; // MAKE SURE content[] is big enough!
  
  // Changed sprintf below to snprintf so we don't crash if the string overflows
  snprintf(content, sizeof(content), "Hello, This is a %s clock. Model %s from www.neonixie.com!<br>"\
  "To configure settings click <a href=/clockconfig>here</a><br>"\
  "Online documentation is available <a href=http://www.neonixie.com/%s/?s=fromaclock>Here</a><br>"\
  "<br>"\
  "Temp C: %.2f Temp F: %.2f Humidity: %.0f<br>"\
  "BMP280 sensor PA: %.2f HG: %.2f Meters: %.2f Feet: %.1f Temperature_C %.1f<br>"\
  "Lux: %.0f<br>"\
  "<br>"\
  "IP = %s<br>"\
  "mDNS Hostname = %s.local<br>"\
  "ARDUINO_BOARD = %s<br>"\
  "Version %s compile time: %s <a href=/partitions.html>Partitions</a><br>"\
  "Uptime Seconds: %lld  Uptime Days: %.2f  Total Runtime Days: %.2f<br>"\
  "%s<br>"\
  , CLOCKTYPE, CLOCKMODEL, CLOCKMODEL, TEMPERATURE_C, TEMPERATURE_F, HUMIDITY, bmp_PA, bmp_HG, bmp_METERS, bmp_FEET, bmp_TEMPERATURE_C, lux, WiFi.localIP().toString().c_str(), tmp_hostname, ARDUINO_BOARD, VERSION, compile_date, esp_timer_get_time()/1000000, UPTIME, TOTAL_RUNTIME, TIME_STRING);
  
  // removed in 0.91 BETA4... "Free memory: %d" ...  ESP.getFreeHeap() 
  Server.send(200, "text/html", content);
}


// Define NTP Client to get time
WiFiUDP ntpUDP;
NTPClient timeClient(ntpUDP, "pool.ntp.org");
//NTPClient timeClient(ntpUDP, "192.168.0.3");

int sec_ones, sec_tens, min_ones, min_tens, hour_ones, hour_tens;
byte sec_ones_dp, sec_tens_dp, min_ones_dp, min_tens_dp, hour_ones_dp, hour_tens_dp;

int display_sec_ones, display_sec_tens, display_min_ones, display_min_tens, display_hour_ones, display_hour_tens;
byte display_sec_ones_dp, display_sec_tens_dp, display_min_ones_dp, display_min_tens_dp, display_hour_ones_dp, display_hour_tens_dp;

int prev_sec_ones, prev_sec_tens, prev_min_ones, prev_min_tens, prev_hour_ones, prev_hour_tens;
byte prev_sec_ones_dp, prev_sec_tens_dp, prev_min_ones_dp, prev_min_tens_dp, prev_hour_ones_dp, prev_hour_tens_dp;
int prev_check_sec_ones, prev_check_sec_tens;

int previous_tm_sec;  // USE THIS TO SEE WHEN A SECOND TICKS OVER

int PREVIOUS_TEMPERATURE_READ_SECONDS;

//AutoConnect portal;
AutoConnectAux  page_add;
AutoConnectAux  page_clockconfigadvanced;
AutoConnectAux  page_alarm;
AutoConnectAux  page_timed_brightness;
//AutoConnectAux  page_results;

int INITIALIZE_WEB_VARIABLES=0;
int INITIALIZE_WEB_VARIABLES2=0;
int INITIALIZE_WEB_VARIABLES3=0;
int INITIALIZE_WEB_VARIABLES4=0;

String onclockconfig(AutoConnectAux& aux, PageArgument& args) {
  // IMPORTANT NOTE!!! IF THE SETTING VARIABLES BELOW ARE NOT ON THE WEBPAGE THE PROGRAM WILL CRASH WHEN YOU TRY TO LOAD THE PAGE!!!
  if ( INITIALIZE_WEB_VARIABLES == 0 ) {
    aux["SETTING_BRIGHTNESS"].as<AutoConnectInput>().value = SETTING_BRIGHTNESS;
    aux["SETTING_LUXMETER"].as<AutoConnectSelect>().selected = SETTING_LUXMETER;
    
    aux["SETTING_TEMP_IN_C"].as<AutoConnectCheckbox>().checked = SETTING_TEMP_IN_C;
    aux["SETTING_TEMPDISP_SEC"].as<AutoConnectInput>().value = SETTING_TEMPDISP_SEC;
    aux["SETTING_TEMPDISP_DUR"].as<AutoConnectInput>().value = SETTING_TEMPDISP_DUR; 
    aux["SETTING_TEMP_COMP"].as<AutoConnectInput>().value = SETTING_TEMP_COMP;
    aux["SETTING_HUMIDITY_COMP"].as<AutoConnectInput>().value = SETTING_HUMIDITY_COMP;
    
    aux["SETTING_12H"].as<AutoConnectCheckbox>().checked = SETTING_12H;
    aux["SETTING_LZBLANK"].as<AutoConnectCheckbox>().checked = SETTING_LZBLANK;
    aux["SETTING_CROSSFADE"].as<AutoConnectCheckbox>().checked = SETTING_CROSSFADE;

    aux["SETTING_DATEFORMAT"].as<AutoConnectSelect>().selected = SETTING_DATEFORMAT;
    aux["SETTING_DATEDISP_SEC"].as<AutoConnectInput>().value = SETTING_DATEDISP_SEC;
    aux["SETTING_DATEDISP_DUR"].as<AutoConnectInput>().value = SETTING_DATEDISP_DUR;

    #ifdef Z57XM6DV2
      aux["SETTING_CATHODEPP"].as<AutoConnectSelect>().selected = SETTING_CATHODEPP;
      aux["SETTING_CATHODEPPSTART"].as<AutoConnectInput>().value = SETTING_CATHODEPPSTART;
      aux["SETTING_CATHODEPPDUR"].as<AutoConnectInput>().value = SETTING_CATHODEPPDUR;
      aux["SETTING_CATHODEPPREV"].as<AutoConnectInput>().value = SETTING_CATHODEPPREV;
      aux["SETTING_CATHODEPPREVD"].as<AutoConnectInput>().value = SETTING_CATHODEPPREVD;
    #endif
    
    aux["SETTING_TIMEZONE"].as<AutoConnectInput>().value = SETTING_TIMEZONE;

    aux["SETTING_COLON"].as<AutoConnectSelect>().selected = SETTING_COLON;
    
    aux["SETTING_BMP"].as<AutoConnectSelect>().selected = SETTING_BMP;

    aux["SETTING_TEMPHUM"].as<AutoConnectSelect>().selected = SETTING_TEMPHUM;
    
    aux["somemoretextbla1"].as<AutoConnectText>().value = "Version " + String(VERSION) + " compile time: " + String(compile_date) + "<br>";   
    INITIALIZE_WEB_VARIABLES = 1;
  } else {

    preferences.begin("clock-settings", false);
    
    SETTING_BRIGHTNESS = aux["SETTING_BRIGHTNESS"].as<AutoConnectInput>().value.toInt();
    preferences.putInt("BRIGHTNESS", SETTING_BRIGHTNESS);
    
    SETTING_LUXMETER = aux["SETTING_LUXMETER"].as<AutoConnectSelect>().selected;
    preferences.putInt("LUXMETER", SETTING_LUXMETER);
    
    SETTING_TEMP_IN_C = aux["SETTING_TEMP_IN_C"].as<AutoConnectCheckbox>().checked;
    preferences.putInt("TEMP_IN_C", SETTING_TEMP_IN_C); 
    SETTING_TEMPDISP_SEC = aux["SETTING_TEMPDISP_SEC"].as<AutoConnectInput>().value.toInt();
    preferences.putInt("TEMPDISP_SEC", SETTING_TEMPDISP_SEC);
    SETTING_TEMPDISP_DUR = aux["SETTING_TEMPDISP_DUR"].as<AutoConnectInput>().value.toInt();
    preferences.putInt("TEMPDISP_DUR", SETTING_TEMPDISP_DUR);  
    
    SETTING_TEMP_COMP = aux["SETTING_TEMP_COMP"].as<AutoConnectInput>().value.toInt();
    preferences.putInt("TEMP_COMP", SETTING_TEMP_COMP);
    SETTING_HUMIDITY_COMP = aux["SETTING_HUMIDITY_COMP"].as<AutoConnectInput>().value.toInt();
    preferences.putInt("HUMIDITY_COMP", SETTING_HUMIDITY_COMP);

    SETTING_12H = aux["SETTING_12H"].as<AutoConnectCheckbox>().checked;
    preferences.putInt("12H", SETTING_12H);

    SETTING_LZBLANK = aux["SETTING_LZBLANK"].as<AutoConnectCheckbox>().checked;
    preferences.putInt("LZBLANK", SETTING_LZBLANK);

    SETTING_CROSSFADE = aux["SETTING_CROSSFADE"].as<AutoConnectCheckbox>().checked;
    preferences.putInt("CROSSFADE", SETTING_CROSSFADE);

    SETTING_DATEFORMAT = aux["SETTING_DATEFORMAT"].as<AutoConnectSelect>().selected;
    preferences.putInt("DATEFORMAT", SETTING_DATEFORMAT);
    SETTING_DATEDISP_SEC = aux["SETTING_DATEDISP_SEC"].as<AutoConnectInput>().value.toInt();
    preferences.putInt("DATEDISP_SEC", SETTING_DATEDISP_SEC);
    SETTING_DATEDISP_DUR = aux["SETTING_DATEDISP_DUR"].as<AutoConnectInput>().value.toInt();
    preferences.putInt("DATEDISP_DUR", SETTING_DATEDISP_DUR);    

    #ifdef Z57XM6DV2
      SETTING_CATHODEPP = aux["SETTING_CATHODEPP"].as<AutoConnectSelect>().selected;
      preferences.putInt("CATHODEPP", SETTING_CATHODEPP);
      SETTING_CATHODEPPSTART = aux["SETTING_CATHODEPPSTART"].as<AutoConnectInput>().value.toInt();
      preferences.putInt("CATHODEPPSTART", SETTING_CATHODEPPSTART);
      SETTING_CATHODEPPDUR = aux["SETTING_CATHODEPPDUR"].as<AutoConnectInput>().value.toInt();
      preferences.putInt("CATHODEPPDUR", SETTING_CATHODEPPDUR);
      SETTING_CATHODEPPREV = aux["SETTING_CATHODEPPREV"].as<AutoConnectInput>().value.toInt();
      preferences.putInt("CATHODEPPREV", SETTING_CATHODEPPREV);
      SETTING_CATHODEPPREVD = aux["SETTING_CATHODEPPREVD"].as<AutoConnectInput>().value.toInt();
      preferences.putInt("CATHODEPPREVD", SETTING_CATHODEPPREVD);
    #endif

    SETTING_TIMEZONE = aux["SETTING_TIMEZONE"].as<AutoConnectInput>().value;
    preferences.putString("TIMEZONE", SETTING_TIMEZONE);

    SETTING_COLON = aux["SETTING_COLON"].as<AutoConnectSelect>().selected;
    preferences.putInt("COLON", SETTING_COLON);

    SETTING_BMP = aux["SETTING_BMP"].as<AutoConnectSelect>().selected;
    preferences.putInt("BMP", SETTING_BMP);

    SETTING_TEMPHUM = aux["SETTING_TEMPHUM"].as<AutoConnectSelect>().selected;
    preferences.putInt("TEMPHUM", SETTING_TEMPHUM);
    
    setTimezone(SETTING_TIMEZONE);
    
    preferences.end();
  }

  return String();
}

String clockconfigadvanced(AutoConnectAux& aux, PageArgument& args) {
  
  if ( INITIALIZE_WEB_VARIABLES2 == 0 ) {
    aux["SETTING_NTPSERVER"].as<AutoConnectInput>().value = SETTING_NTPSERVER;
    aux["SETTING_MQTTENABLE"].as<AutoConnectCheckbox>().checked = SETTING_MQTTENABLE;
    aux["SETTING_MQTTBRIGHT"].as<AutoConnectCheckbox>().checked = SETTING_MQTTBRIGHT;
    aux["SETTING_MQTTSERVER"].as<AutoConnectInput>().value = SETTING_MQTTSERVER;
    aux["SETTING_MQTTPORT"].as<AutoConnectInput>().value = SETTING_MQTTPORT;
    aux["SETTING_MQTTUSERNAME"].as<AutoConnectInput>().value = SETTING_MQTTUSERNAME;
    aux["SETTING_MQTTPASSWORD"].as<AutoConnectInput>().value = SETTING_MQTTPASSWORD;
    aux["SETTING_MQTTROOT"].as<AutoConnectInput>().value = SETTING_MQTTROOT;

    aux["SETTING_MQTTTEMP_F"].as<AutoConnectCheckbox>().checked = SETTING_MQTTTEMP_F;
    aux["SETTING_MQTTTEMP_C"].as<AutoConnectCheckbox>().checked = SETTING_MQTTTEMP_C;
    aux["SETTING_MQTTHUM"].as<AutoConnectCheckbox>().checked = SETTING_MQTTHUM;
    aux["SETTING_MQTTLUX"].as<AutoConnectCheckbox>().checked = SETTING_MQTTLUX;
    aux["SETTING_MQTTbmp_PA"].as<AutoConnectCheckbox>().checked = SETTING_MQTTbmp_PA;
    aux["SETTING_MQTTbmp_HG"].as<AutoConnectCheckbox>().checked = SETTING_MQTTbmp_HG;
    aux["SETTING_MQTTbmp_METERS"].as<AutoConnectCheckbox>().checked = SETTING_MQTTbmp_METERS;
    aux["SETTING_MQTTbmp_FEET"].as<AutoConnectCheckbox>().checked = SETTING_MQTTbmp_FEET;
    aux["SETTING_MQTTbmp_TEMP_C"].as<AutoConnectCheckbox>().checked = SETTING_MQTTbmp_TEMP_C;
    aux["SETTING_MQTTRSSI"].as<AutoConnectCheckbox>().checked = SETTING_MQTTRSSI;

    #ifdef IV_9_6D_RR
      aux["SETTING_TUBEONBOTTOM"].as<AutoConnectSelect>().selected = SETTING_TUBEONBOTTOM;
      aux["SETTING_KEEPWARMRATIO"].as<AutoConnectInput>().value = SETTING_KEEPWARMRATIO;
    #endif

    aux["SETTING_ALTDISPLAY"].as<AutoConnectSelect>().selected = SETTING_ALTDISPLAY;

    aux["SETTING_MINBRIGHTNESS"].as<AutoConnectInput>().value = SETTING_MINBRIGHTNESS;

    aux["SETTING_QUICKBOOT"].as<AutoConnectCheckbox>().checked = SETTING_QUICKBOOT;
    
    aux["web_tmp_hostname"].as<AutoConnectText>().value = tmp_hostname;   

    INITIALIZE_WEB_VARIABLES2 = 1;
  } else {
    preferences.begin("clock-settings", false);
    SETTING_NTPSERVER = aux["SETTING_NTPSERVER"].as<AutoConnectInput>().value;
    preferences.putString("NTPSERVER", SETTING_NTPSERVER);
    SETTING_MQTTENABLE = aux["SETTING_MQTTENABLE"].as<AutoConnectCheckbox>().checked;
    preferences.putInt("MQTTENABLE", SETTING_MQTTENABLE);
    SETTING_MQTTBRIGHT = aux["SETTING_MQTTBRIGHT"].as<AutoConnectCheckbox>().checked;
    preferences.putInt("MQTTBRIGHT", SETTING_MQTTBRIGHT);
    SETTING_MQTTSERVER = aux["SETTING_MQTTSERVER"].as<AutoConnectInput>().value;
    preferences.putString("MQTTSERVER", SETTING_MQTTSERVER);
    SETTING_MQTTPORT = aux["SETTING_MQTTPORT"].as<AutoConnectInput>().value;
    preferences.putString("MQTTPORT", SETTING_MQTTPORT);
    SETTING_MQTTUSERNAME = aux["SETTING_MQTTUSERNAME"].as<AutoConnectInput>().value;
    preferences.putString("MQTTUSERNAME", SETTING_MQTTUSERNAME);
    SETTING_MQTTPASSWORD = aux["SETTING_MQTTPASSWORD"].as<AutoConnectInput>().value;
    preferences.putString("MQTTPASSWORD", SETTING_MQTTPASSWORD);
    SETTING_MQTTROOT = aux["SETTING_MQTTROOT"].as<AutoConnectInput>().value;
    preferences.putString("MQTTROOT", SETTING_MQTTROOT);

    SETTING_MQTTTEMP_F = aux["SETTING_MQTTTEMP_F"].as<AutoConnectCheckbox>().checked;
    preferences.putInt("MQTTTEMP_F", SETTING_MQTTTEMP_F);
    SETTING_MQTTTEMP_C = aux["SETTING_MQTTTEMP_C"].as<AutoConnectCheckbox>().checked;
    preferences.putInt("MQTTTEMP_C", SETTING_MQTTTEMP_C);
    SETTING_MQTTLUX = aux["SETTING_MQTTLUX"].as<AutoConnectCheckbox>().checked;
    preferences.putInt("MQTTLUX", SETTING_MQTTLUX);
    SETTING_MQTTHUM = aux["SETTING_MQTTHUM"].as<AutoConnectCheckbox>().checked;
    preferences.putInt("MQTTHUM", SETTING_MQTTHUM);
    SETTING_MQTTbmp_PA = aux["SETTING_MQTTbmp_PA"].as<AutoConnectCheckbox>().checked;
    preferences.putInt("MQTTbmp_PA", SETTING_MQTTbmp_PA);
    SETTING_MQTTbmp_HG = aux["SETTING_MQTTbmp_HG"].as<AutoConnectCheckbox>().checked;
    preferences.putInt("MQTTbmp_HG", SETTING_MQTTbmp_HG);
    SETTING_MQTTbmp_METERS = aux["SETTING_MQTTbmp_METERS"].as<AutoConnectCheckbox>().checked;
    preferences.putInt("MQTTbmp_METERS", SETTING_MQTTbmp_METERS);
    SETTING_MQTTbmp_FEET = aux["SETTING_MQTTbmp_FEET"].as<AutoConnectCheckbox>().checked;
    preferences.putInt("MQTTbmp_FEET", SETTING_MQTTbmp_FEET);
    SETTING_MQTTbmp_TEMP_C = aux["SETTING_MQTTbmp_TEMP_C"].as<AutoConnectCheckbox>().checked;
    preferences.putInt("MQTTbmp_TEMP_C", SETTING_MQTTbmp_TEMP_C);
    SETTING_MQTTRSSI = aux["SETTING_MQTTRSSI"].as<AutoConnectCheckbox>().checked;
    preferences.putInt("MQTTRSSI", SETTING_MQTTRSSI);

    #ifdef IV_9_6D_RR 
      SETTING_TUBEONBOTTOM = aux["SETTING_TUBEONBOTTOM"].as<AutoConnectSelect>().selected;
      preferences.putInt("TUBEONBOTTOM", SETTING_TUBEONBOTTOM);
      SETTING_KEEPWARMRATIO = aux["SETTING_KEEPWARMRATIO"].as<AutoConnectInput>().value.toInt();
      preferences.putInt("KEEPWARMRATIO", SETTING_KEEPWARMRATIO);  
    #endif

    SETTING_ALTDISPLAY = aux["SETTING_ALTDISPLAY"].as<AutoConnectSelect>().selected;
    preferences.putInt("ALTDISPLAY", SETTING_ALTDISPLAY);

    SETTING_MINBRIGHTNESS = aux["SETTING_MINBRIGHTNESS"].as<AutoConnectInput>().value.toInt();
    preferences.putInt("MINBRIGHTNESS", SETTING_MINBRIGHTNESS);

    SETTING_QUICKBOOT = aux["SETTING_QUICKBOOT"].as<AutoConnectSelect>().selected;
    preferences.putInt("QUICKBOOT", SETTING_QUICKBOOT);

    SETTING_MQTTENABLE = aux["SETTING_MQTTENABLE"].as<AutoConnectCheckbox>().checked;
    preferences.putInt("MQTTENABLE", SETTING_MQTTENABLE);
    
    preferences.end();
  }

  timeClient.setPoolServerName(SETTING_NTPSERVER.c_str());
 
  return String();
}

String alarmfunction(AutoConnectAux& aux, PageArgument& args) {
  if ( INITIALIZE_WEB_VARIABLES3 == 0 ) {
    aux["SETTING_ALARM1ENABLE"].as<AutoConnectCheckbox>().checked = SETTING_ALARM1ENABLE;  
    aux["SETTING_ALARM1SUN"].as<AutoConnectCheckbox>().checked = SETTING_ALARM1SUN;  
    aux["SETTING_ALARM1MON"].as<AutoConnectCheckbox>().checked = SETTING_ALARM1MON;  
    aux["SETTING_ALARM1TUE"].as<AutoConnectCheckbox>().checked = SETTING_ALARM1TUE;  
    aux["SETTING_ALARM1WED"].as<AutoConnectCheckbox>().checked = SETTING_ALARM1WED;  
    aux["SETTING_ALARM1THU"].as<AutoConnectCheckbox>().checked = SETTING_ALARM1THU;  
    aux["SETTING_ALARM1FRI"].as<AutoConnectCheckbox>().checked = SETTING_ALARM1FRI;  
    aux["SETTING_ALARM1SAT"].as<AutoConnectCheckbox>().checked = SETTING_ALARM1SAT; 
    aux["SETTING_ALARM1MONTH"].as<AutoConnectSelect>().selected = SETTING_ALARM1MONTH;
    aux["SETTING_ALARM1DAY"].as<AutoConnectSelect>().selected = SETTING_ALARM1DAY; 
    aux["SETTING_ALARM1HOUR"].as<AutoConnectSelect>().selected = SETTING_ALARM1HOUR+1;  // THE HTML FORM PULLDOWN FOR THESE VALUES STARTS AT "1"... so to keep the number correct we add 1 when we pass it to the HTML page
    aux["SETTING_ALARM1MIN"].as<AutoConnectSelect>().selected = SETTING_ALARM1MIN+1; // THE HTML FORM PULLDOWN FOR THESE VALUES STARTS AT "1"... so to keep the number correct we add 1 when we pass it to the HTML page
    aux["SETTING_ALARM1DUR"].as<AutoConnectSelect>().selected = SETTING_ALARM1DUR;
    aux["SETTING_ALARM1TUNE"].as<AutoConnectSelect>().selected = SETTING_ALARM1TUNE;
    
    aux["SETTING_ALARM2ENABLE"].as<AutoConnectCheckbox>().checked = SETTING_ALARM2ENABLE;  
    aux["SETTING_ALARM2SUN"].as<AutoConnectCheckbox>().checked = SETTING_ALARM2SUN;  
    aux["SETTING_ALARM2MON"].as<AutoConnectCheckbox>().checked = SETTING_ALARM2MON;  
    aux["SETTING_ALARM2TUE"].as<AutoConnectCheckbox>().checked = SETTING_ALARM2TUE;  
    aux["SETTING_ALARM2WED"].as<AutoConnectCheckbox>().checked = SETTING_ALARM2WED;  
    aux["SETTING_ALARM2THU"].as<AutoConnectCheckbox>().checked = SETTING_ALARM2THU;  
    aux["SETTING_ALARM2FRI"].as<AutoConnectCheckbox>().checked = SETTING_ALARM2FRI;  
    aux["SETTING_ALARM2SAT"].as<AutoConnectCheckbox>().checked = SETTING_ALARM2SAT;  
    aux["SETTING_ALARM2MONTH"].as<AutoConnectSelect>().selected = SETTING_ALARM2MONTH;
    aux["SETTING_ALARM2DAY"].as<AutoConnectSelect>().selected = SETTING_ALARM2DAY; 
    aux["SETTING_ALARM2HOUR"].as<AutoConnectSelect>().selected = SETTING_ALARM2HOUR+1;  // THE HTML FORM PULLDOWN FOR THESE VALUES STARTS AT "1"... so to keep the number correct we add 1 when we pass it to the HTML page
    aux["SETTING_ALARM2MIN"].as<AutoConnectSelect>().selected = SETTING_ALARM2MIN+1; // THE HTML FORM PULLDOWN FOR THESE VALUES STARTS AT "1"... so to keep the number correct we add 1 when we pass it to the HTML page
    aux["SETTING_ALARM2DUR"].as<AutoConnectSelect>().selected = SETTING_ALARM2DUR;
    aux["SETTING_ALARM2TUNE"].as<AutoConnectSelect>().selected = SETTING_ALARM2TUNE;

    aux["SETTING_ALARM3ENABLE"].as<AutoConnectCheckbox>().checked = SETTING_ALARM3ENABLE;  
    aux["SETTING_ALARM3SUN"].as<AutoConnectCheckbox>().checked = SETTING_ALARM3SUN;  
    aux["SETTING_ALARM3MON"].as<AutoConnectCheckbox>().checked = SETTING_ALARM3MON;  
    aux["SETTING_ALARM3TUE"].as<AutoConnectCheckbox>().checked = SETTING_ALARM3TUE;  
    aux["SETTING_ALARM3WED"].as<AutoConnectCheckbox>().checked = SETTING_ALARM3WED;  
    aux["SETTING_ALARM3THU"].as<AutoConnectCheckbox>().checked = SETTING_ALARM3THU;  
    aux["SETTING_ALARM3FRI"].as<AutoConnectCheckbox>().checked = SETTING_ALARM3FRI;  
    aux["SETTING_ALARM3SAT"].as<AutoConnectCheckbox>().checked = SETTING_ALARM3SAT;  
    aux["SETTING_ALARM3MONTH"].as<AutoConnectSelect>().selected = SETTING_ALARM3MONTH;
    aux["SETTING_ALARM3DAY"].as<AutoConnectSelect>().selected = SETTING_ALARM3DAY; 
    aux["SETTING_ALARM3HOUR"].as<AutoConnectSelect>().selected = SETTING_ALARM3HOUR+1;  // THE HTML FORM PULLDOWN FOR THESE VALUES STARTS AT "1"... so to keep the number correct we add 1 when we pass it to the HTML page
    aux["SETTING_ALARM3MIN"].as<AutoConnectSelect>().selected = SETTING_ALARM3MIN+1; // THE HTML FORM PULLDOWN FOR THESE VALUES STARTS AT "1"... so to keep the number correct we add 1 when we pass it to the HTML page
    aux["SETTING_ALARM3DUR"].as<AutoConnectSelect>().selected = SETTING_ALARM3DUR;
    aux["SETTING_ALARM3TUNE"].as<AutoConnectSelect>().selected = SETTING_ALARM3TUNE;

    aux["SETTING_RTTTL1"].as<AutoConnectInput>().value = SETTING_RTTTL1;
    aux["SETTING_RTTTL2"].as<AutoConnectInput>().value = SETTING_RTTTL2;
    aux["SETTING_RTTTL3"].as<AutoConnectInput>().value = SETTING_RTTTL3;

    INITIALIZE_WEB_VARIABLES3 = 1;
  } else {
    preferences.begin("clock-settings", false);
    SETTING_ALARM1ENABLE = aux["SETTING_ALARM1ENABLE"].as<AutoConnectCheckbox>().checked;
    preferences.putInt("ALARM1ENABLE", SETTING_ALARM1ENABLE);
    SETTING_ALARM1SUN = aux["SETTING_ALARM1SUN"].as<AutoConnectCheckbox>().checked;
    preferences.putInt("ALARM1SUN", SETTING_ALARM1SUN);
    SETTING_ALARM1MON = aux["SETTING_ALARM1MON"].as<AutoConnectCheckbox>().checked;
    preferences.putInt("ALARM1MON", SETTING_ALARM1MON);
    SETTING_ALARM1TUE = aux["SETTING_ALARM1TUE"].as<AutoConnectCheckbox>().checked;
    preferences.putInt("ALARM1TUE", SETTING_ALARM1TUE);
    SETTING_ALARM1WED = aux["SETTING_ALARM1WED"].as<AutoConnectCheckbox>().checked;
    preferences.putInt("ALARM1WED", SETTING_ALARM1WED);
    SETTING_ALARM1THU = aux["SETTING_ALARM1THU"].as<AutoConnectCheckbox>().checked;
    preferences.putInt("ALARM1THU", SETTING_ALARM1THU);
    SETTING_ALARM1FRI = aux["SETTING_ALARM1FRI"].as<AutoConnectCheckbox>().checked;
    preferences.putInt("ALARM1FRI", SETTING_ALARM1FRI);
    SETTING_ALARM1SAT = aux["SETTING_ALARM1SAT"].as<AutoConnectCheckbox>().checked;
    preferences.putInt("ALARM1SAT", SETTING_ALARM1SAT);
    SETTING_ALARM1MONTH = aux["SETTING_ALARM1MONTH"].as<AutoConnectSelect>().selected;
    preferences.putInt("ALARM1MONTH", SETTING_ALARM1MONTH);
    SETTING_ALARM1DAY = aux["SETTING_ALARM1DAY"].as<AutoConnectSelect>().selected;
    preferences.putInt("ALARM1DAY", SETTING_ALARM1DAY);
    SETTING_ALARM1HOUR = aux["SETTING_ALARM1HOUR"].as<AutoConnectSelect>().selected - 1; // THE HTML FORM PULLDOWN FOR THESE VALUES STARTS AT "1"... so to keep the number correct we subtract 1 when we store it
    preferences.putInt("ALARM1HOUR", SETTING_ALARM1HOUR);
    SETTING_ALARM1MIN = aux["SETTING_ALARM1MIN"].as<AutoConnectSelect>().selected - 1; // THE HTML FORM PULLDOWN FOR THESE VALUES STARTS AT "1"... so to keep the number correct we subtract 1 when we store it
    preferences.putInt("ALARM1MIN", SETTING_ALARM1MIN);
    SETTING_ALARM1DUR = aux["SETTING_ALARM1DUR"].as<AutoConnectSelect>().selected;
    preferences.putInt("ALARM1DUR", SETTING_ALARM1DUR);
    SETTING_ALARM1TUNE = aux["SETTING_ALARM1TUNE"].as<AutoConnectSelect>().selected;
    preferences.putInt("ALARM1TUNE", SETTING_ALARM1TUNE);

    SETTING_ALARM2ENABLE = aux["SETTING_ALARM2ENABLE"].as<AutoConnectCheckbox>().checked;
    preferences.putInt("ALARM2ENABLE", SETTING_ALARM2ENABLE);
    SETTING_ALARM2SUN = aux["SETTING_ALARM2SUN"].as<AutoConnectCheckbox>().checked;
    preferences.putInt("ALARM2SUN", SETTING_ALARM2SUN);
    SETTING_ALARM2MON = aux["SETTING_ALARM2MON"].as<AutoConnectCheckbox>().checked;
    preferences.putInt("ALARM2MON", SETTING_ALARM2MON);
    SETTING_ALARM2TUE = aux["SETTING_ALARM2TUE"].as<AutoConnectCheckbox>().checked;
    preferences.putInt("ALARM2TUE", SETTING_ALARM2TUE);
    SETTING_ALARM2WED = aux["SETTING_ALARM2WED"].as<AutoConnectCheckbox>().checked;
    preferences.putInt("ALARM2WED", SETTING_ALARM2WED);
    SETTING_ALARM2THU = aux["SETTING_ALARM2THU"].as<AutoConnectCheckbox>().checked;
    preferences.putInt("ALARM2THU", SETTING_ALARM2THU);
    SETTING_ALARM2FRI = aux["SETTING_ALARM2FRI"].as<AutoConnectCheckbox>().checked;
    preferences.putInt("ALARM2FRI", SETTING_ALARM2FRI);
    SETTING_ALARM2SAT = aux["SETTING_ALARM2SAT"].as<AutoConnectCheckbox>().checked;
    preferences.putInt("ALARM2SAT", SETTING_ALARM2SAT);
    SETTING_ALARM2MONTH = aux["SETTING_ALARM2MONTH"].as<AutoConnectSelect>().selected;
    preferences.putInt("ALARM2MONTH", SETTING_ALARM2MONTH);
    SETTING_ALARM2DAY = aux["SETTING_ALARM2DAY"].as<AutoConnectSelect>().selected;
    preferences.putInt("ALARM2DAY", SETTING_ALARM2DAY);
    SETTING_ALARM2HOUR = aux["SETTING_ALARM2HOUR"].as<AutoConnectSelect>().selected - 1; // THE HTML FORM PULLDOWN FOR THESE VALUES STARTS AT "1"... so to keep the number correct we subtract 1 when we store it
    preferences.putInt("ALARM2HOUR", SETTING_ALARM2HOUR);
    SETTING_ALARM2MIN = aux["SETTING_ALARM2MIN"].as<AutoConnectSelect>().selected - 1; // THE HTML FORM PULLDOWN FOR THESE VALUES STARTS AT "1"... so to keep the number correct we subtract 1 when we store it
    preferences.putInt("ALARM2MIN", SETTING_ALARM2MIN);
    SETTING_ALARM2DUR = aux["SETTING_ALARM2DUR"].as<AutoConnectSelect>().selected;
    preferences.putInt("ALARM2DUR", SETTING_ALARM2DUR);
    SETTING_ALARM2TUNE = aux["SETTING_ALARM2TUNE"].as<AutoConnectSelect>().selected;
    preferences.putInt("ALARM2TUNE", SETTING_ALARM2TUNE);

    SETTING_ALARM3ENABLE = aux["SETTING_ALARM3ENABLE"].as<AutoConnectCheckbox>().checked;
    preferences.putInt("ALARM3ENABLE", SETTING_ALARM3ENABLE);
    SETTING_ALARM3SUN = aux["SETTING_ALARM3SUN"].as<AutoConnectCheckbox>().checked;
    preferences.putInt("ALARM3SUN", SETTING_ALARM3SUN);
    SETTING_ALARM3MON = aux["SETTING_ALARM3MON"].as<AutoConnectCheckbox>().checked;
    preferences.putInt("ALARM3MON", SETTING_ALARM3MON);
    SETTING_ALARM3TUE = aux["SETTING_ALARM3TUE"].as<AutoConnectCheckbox>().checked;
    preferences.putInt("ALARM3TUE", SETTING_ALARM3TUE);
    SETTING_ALARM3WED = aux["SETTING_ALARM3WED"].as<AutoConnectCheckbox>().checked;
    preferences.putInt("ALARM3WED", SETTING_ALARM3WED);
    SETTING_ALARM3THU = aux["SETTING_ALARM3THU"].as<AutoConnectCheckbox>().checked;
    preferences.putInt("ALARM3THU", SETTING_ALARM3THU);
    SETTING_ALARM3FRI = aux["SETTING_ALARM3FRI"].as<AutoConnectCheckbox>().checked;
    preferences.putInt("ALARM3FRI", SETTING_ALARM3FRI);
    SETTING_ALARM3SAT = aux["SETTING_ALARM3SAT"].as<AutoConnectCheckbox>().checked;
    preferences.putInt("ALARM3SAT", SETTING_ALARM3SAT);
    SETTING_ALARM3MONTH = aux["SETTING_ALARM3MONTH"].as<AutoConnectSelect>().selected;
    preferences.putInt("ALARM3MONTH", SETTING_ALARM3MONTH);
    SETTING_ALARM3DAY = aux["SETTING_ALARM3DAY"].as<AutoConnectSelect>().selected;
    preferences.putInt("ALARM3DAY", SETTING_ALARM3DAY);
    SETTING_ALARM3HOUR = aux["SETTING_ALARM3HOUR"].as<AutoConnectSelect>().selected - 1; // THE HTML FORM PULLDOWN FOR THESE VALUES STARTS AT "1"... so to keep the number correct we subtract 1 when we store it
    preferences.putInt("ALARM3HOUR", SETTING_ALARM3HOUR);
    SETTING_ALARM3MIN = aux["SETTING_ALARM3MIN"].as<AutoConnectSelect>().selected - 1; // THE HTML FORM PULLDOWN FOR THESE VALUES STARTS AT "1"... so to keep the number correct we subtract 1 when we store it
    preferences.putInt("ALARM3MIN", SETTING_ALARM3MIN);
    SETTING_ALARM3DUR = aux["SETTING_ALARM3DUR"].as<AutoConnectSelect>().selected;
    preferences.putInt("ALARM3DUR", SETTING_ALARM3DUR);
    SETTING_ALARM3TUNE = aux["SETTING_ALARM3TUNE"].as<AutoConnectSelect>().selected;
    preferences.putInt("ALARM3TUNE", SETTING_ALARM3TUNE);

    SETTING_RTTTL1 = aux["SETTING_RTTTL1"].as<AutoConnectInput>().value;
    preferences.putString("RTTTL1", SETTING_RTTTL1);
    SETTING_RTTTL2 = aux["SETTING_RTTTL2"].as<AutoConnectInput>().value;
    preferences.putString("RTTTL2", SETTING_RTTTL2);
    SETTING_RTTTL3 = aux["SETTING_RTTTL3"].as<AutoConnectInput>().value;
    preferences.putString("RTTTL3", SETTING_RTTTL3);

    SETTING_ONETIMEPLAY = aux["SETTING_ONETIMEPLAY"].as<AutoConnectSelect>().selected;  

    // IF WE HAVE A MONTH OR DAY SPECIFIED.. WE CLEAR CLEAR ALL THE DAY OF WEEK CHECKBOXES SINCE THAT DON'T MAKE SENSE!
    if ( SETTING_ALARM1MONTH != 13 || SETTING_ALARM1DAY != 32 ) {
      SETTING_ALARM1SUN = 0;  aux["SETTING_ALARM1SUN"].as<AutoConnectCheckbox>().checked = SETTING_ALARM1SUN; preferences.putInt("ALARM1SUN", SETTING_ALARM1SUN);
      SETTING_ALARM1MON = 0;  aux["SETTING_ALARM1MON"].as<AutoConnectCheckbox>().checked = SETTING_ALARM1MON; preferences.putInt("ALARM1MON", SETTING_ALARM1MON);
      SETTING_ALARM1TUE = 0;  aux["SETTING_ALARM1TUE"].as<AutoConnectCheckbox>().checked = SETTING_ALARM1TUE; preferences.putInt("ALARM1TUE", SETTING_ALARM1TUE);
      SETTING_ALARM1WED = 0;  aux["SETTING_ALARM1WED"].as<AutoConnectCheckbox>().checked = SETTING_ALARM1WED; preferences.putInt("ALARM1WED", SETTING_ALARM1WED);
      SETTING_ALARM1THU = 0;  aux["SETTING_ALARM1THU"].as<AutoConnectCheckbox>().checked = SETTING_ALARM1THU; preferences.putInt("ALARM1THU", SETTING_ALARM1THU);
      SETTING_ALARM1FRI = 0;  aux["SETTING_ALARM1FRI"].as<AutoConnectCheckbox>().checked = SETTING_ALARM1FRI; preferences.putInt("ALARM1FRI", SETTING_ALARM1FRI);
      SETTING_ALARM1SAT = 0;  aux["SETTING_ALARM1SAT"].as<AutoConnectCheckbox>().checked = SETTING_ALARM1SAT; preferences.putInt("ALARM1SAT", SETTING_ALARM1SAT);
    }

    if ( SETTING_ALARM2MONTH != 13 || SETTING_ALARM2DAY != 32 ) {
      SETTING_ALARM2SUN = 0;  aux["SETTING_ALARM2SUN"].as<AutoConnectCheckbox>().checked = SETTING_ALARM2SUN; preferences.putInt("ALARM2SUN", SETTING_ALARM2SUN);
      SETTING_ALARM2MON = 0;  aux["SETTING_ALARM2MON"].as<AutoConnectCheckbox>().checked = SETTING_ALARM2MON; preferences.putInt("ALARM2MON", SETTING_ALARM2MON);
      SETTING_ALARM2TUE = 0;  aux["SETTING_ALARM2TUE"].as<AutoConnectCheckbox>().checked = SETTING_ALARM2TUE; preferences.putInt("ALARM2TUE", SETTING_ALARM2TUE);
      SETTING_ALARM2WED = 0;  aux["SETTING_ALARM2WED"].as<AutoConnectCheckbox>().checked = SETTING_ALARM2WED; preferences.putInt("ALARM2WED", SETTING_ALARM2WED);
      SETTING_ALARM2THU = 0;  aux["SETTING_ALARM2THU"].as<AutoConnectCheckbox>().checked = SETTING_ALARM2THU; preferences.putInt("ALARM2THU", SETTING_ALARM2THU);
      SETTING_ALARM2FRI = 0;  aux["SETTING_ALARM2FRI"].as<AutoConnectCheckbox>().checked = SETTING_ALARM2FRI; preferences.putInt("ALARM2FRI", SETTING_ALARM2FRI);
      SETTING_ALARM2SAT = 0;  aux["SETTING_ALARM2SAT"].as<AutoConnectCheckbox>().checked = SETTING_ALARM2SAT; preferences.putInt("ALARM2SAT", SETTING_ALARM2SAT);
    }

    if ( SETTING_ALARM3MONTH != 13 || SETTING_ALARM3DAY != 32 ) {
      SETTING_ALARM3SUN = 0;  aux["SETTING_ALARM3SUN"].as<AutoConnectCheckbox>().checked = SETTING_ALARM3SUN; preferences.putInt("ALARM3SUN", SETTING_ALARM3SUN);
      SETTING_ALARM3MON = 0;  aux["SETTING_ALARM3MON"].as<AutoConnectCheckbox>().checked = SETTING_ALARM3MON; preferences.putInt("ALARM3MON", SETTING_ALARM3MON);
      SETTING_ALARM3TUE = 0;  aux["SETTING_ALARM3TUE"].as<AutoConnectCheckbox>().checked = SETTING_ALARM3TUE; preferences.putInt("ALARM3TUE", SETTING_ALARM3TUE);
      SETTING_ALARM3WED = 0;  aux["SETTING_ALARM3WED"].as<AutoConnectCheckbox>().checked = SETTING_ALARM3WED; preferences.putInt("ALARM3WED", SETTING_ALARM3WED);
      SETTING_ALARM3THU = 0;  aux["SETTING_ALARM3THU"].as<AutoConnectCheckbox>().checked = SETTING_ALARM3THU; preferences.putInt("ALARM3THU", SETTING_ALARM3THU);
      SETTING_ALARM3FRI = 0;  aux["SETTING_ALARM3FRI"].as<AutoConnectCheckbox>().checked = SETTING_ALARM3FRI; preferences.putInt("ALARM3FRI", SETTING_ALARM3FRI);
      SETTING_ALARM3SAT = 0;  aux["SETTING_ALARM3SAT"].as<AutoConnectCheckbox>().checked = SETTING_ALARM3SAT; preferences.putInt("ALARM3SAT", SETTING_ALARM3SAT);
    }

    
    preferences.end();
  }

  
  ALARM1_DURATION=88888;    // THESE STOP ANY CURRENT ALARM IN PROGRESS
  ALARM2_DURATION=88888;
  ALARM3_DURATION=88888;
  return String();
}

String timed_brightness(AutoConnectAux& aux, PageArgument& args) {
  if ( INITIALIZE_WEB_VARIABLES4 == 0 ) {
    aux["SETTING_TB1ENABLE"].as<AutoConnectCheckbox>().checked = SETTING_TB1ENABLE;  
    aux["SETTING_TB1BRIGHT"].as<AutoConnectInput>().value = SETTING_TB1BRIGHT;
    aux["SETTING_TB1SUN"].as<AutoConnectCheckbox>().checked = SETTING_TB1SUN;  
    aux["SETTING_TB1MON"].as<AutoConnectCheckbox>().checked = SETTING_TB1MON;  
    aux["SETTING_TB1TUE"].as<AutoConnectCheckbox>().checked = SETTING_TB1TUE;  
    aux["SETTING_TB1WED"].as<AutoConnectCheckbox>().checked = SETTING_TB1WED;  
    aux["SETTING_TB1THU"].as<AutoConnectCheckbox>().checked = SETTING_TB1THU;  
    aux["SETTING_TB1FRI"].as<AutoConnectCheckbox>().checked = SETTING_TB1FRI;  
    aux["SETTING_TB1SAT"].as<AutoConnectCheckbox>().checked = SETTING_TB1SAT;  
    aux["SETTING_TB1FROM"].as<AutoConnectInput>().value = SETTING_TB1FROM;
    aux["SETTING_TB1TO"].as<AutoConnectInput>().value = SETTING_TB1TO;

    aux["SETTING_TB2ENABLE"].as<AutoConnectCheckbox>().checked = SETTING_TB2ENABLE;  
    aux["SETTING_TB2BRIGHT"].as<AutoConnectInput>().value = SETTING_TB2BRIGHT;
    aux["SETTING_TB2SUN"].as<AutoConnectCheckbox>().checked = SETTING_TB2SUN;  
    aux["SETTING_TB2MON"].as<AutoConnectCheckbox>().checked = SETTING_TB2MON;  
    aux["SETTING_TB2TUE"].as<AutoConnectCheckbox>().checked = SETTING_TB2TUE;  
    aux["SETTING_TB2WED"].as<AutoConnectCheckbox>().checked = SETTING_TB2WED;  
    aux["SETTING_TB2THU"].as<AutoConnectCheckbox>().checked = SETTING_TB2THU;  
    aux["SETTING_TB2FRI"].as<AutoConnectCheckbox>().checked = SETTING_TB2FRI;  
    aux["SETTING_TB2SAT"].as<AutoConnectCheckbox>().checked = SETTING_TB2SAT;  
    aux["SETTING_TB2FROM"].as<AutoConnectInput>().value = SETTING_TB2FROM;
    aux["SETTING_TB2TO"].as<AutoConnectInput>().value = SETTING_TB2TO;
    
    aux["SETTING_TB3ENABLE"].as<AutoConnectCheckbox>().checked = SETTING_TB3ENABLE;  
    aux["SETTING_TB3BRIGHT"].as<AutoConnectInput>().value = SETTING_TB3BRIGHT;
    aux["SETTING_TB3SUN"].as<AutoConnectCheckbox>().checked = SETTING_TB3SUN;  
    aux["SETTING_TB3MON"].as<AutoConnectCheckbox>().checked = SETTING_TB3MON;  
    aux["SETTING_TB3TUE"].as<AutoConnectCheckbox>().checked = SETTING_TB3TUE;  
    aux["SETTING_TB3WED"].as<AutoConnectCheckbox>().checked = SETTING_TB3WED;  
    aux["SETTING_TB3THU"].as<AutoConnectCheckbox>().checked = SETTING_TB3THU;  
    aux["SETTING_TB3FRI"].as<AutoConnectCheckbox>().checked = SETTING_TB3FRI;  
    aux["SETTING_TB3SAT"].as<AutoConnectCheckbox>().checked = SETTING_TB3SAT;  
    aux["SETTING_TB3FROM"].as<AutoConnectInput>().value = SETTING_TB3FROM;
    aux["SETTING_TB3TO"].as<AutoConnectInput>().value = SETTING_TB3TO;

    aux["SETTING_TB4ENABLE"].as<AutoConnectCheckbox>().checked = SETTING_TB4ENABLE;  
    aux["SETTING_TB4BRIGHT"].as<AutoConnectInput>().value = SETTING_TB4BRIGHT;
    aux["SETTING_TB4SUN"].as<AutoConnectCheckbox>().checked = SETTING_TB4SUN;  
    aux["SETTING_TB4MON"].as<AutoConnectCheckbox>().checked = SETTING_TB4MON;  
    aux["SETTING_TB4TUE"].as<AutoConnectCheckbox>().checked = SETTING_TB4TUE;  
    aux["SETTING_TB4WED"].as<AutoConnectCheckbox>().checked = SETTING_TB4WED;  
    aux["SETTING_TB4THU"].as<AutoConnectCheckbox>().checked = SETTING_TB4THU;  
    aux["SETTING_TB4FRI"].as<AutoConnectCheckbox>().checked = SETTING_TB4FRI;  
    aux["SETTING_TB4SAT"].as<AutoConnectCheckbox>().checked = SETTING_TB4SAT;  
    aux["SETTING_TB4FROM"].as<AutoConnectInput>().value = SETTING_TB4FROM;
    aux["SETTING_TB4TO"].as<AutoConnectInput>().value = SETTING_TB4TO;
    
    INITIALIZE_WEB_VARIABLES4 = 1;
  } else {
    preferences.begin("clock-settings", false);
    SETTING_TB1ENABLE = aux["SETTING_TB1ENABLE"].as<AutoConnectCheckbox>().checked;
    preferences.putInt("TB1ENABLE", SETTING_TB1ENABLE);
    SETTING_TB1BRIGHT = aux["SETTING_TB1BRIGHT"].as<AutoConnectInput>().value.toInt();
    preferences.putInt("TB1BRIGHT", SETTING_TB1BRIGHT);
    SETTING_TB1SUN = aux["SETTING_TB1SUN"].as<AutoConnectCheckbox>().checked;
    preferences.putInt("TB1SUN", SETTING_TB1SUN);
    SETTING_TB1MON = aux["SETTING_TB1MON"].as<AutoConnectCheckbox>().checked;
    preferences.putInt("TB1MON", SETTING_TB1MON);
    SETTING_TB1TUE = aux["SETTING_TB1TUE"].as<AutoConnectCheckbox>().checked;
    preferences.putInt("TB1TUE", SETTING_TB1TUE);
    SETTING_TB1WED = aux["SETTING_TB1WED"].as<AutoConnectCheckbox>().checked;
    preferences.putInt("TB1WED", SETTING_TB1WED);
    SETTING_TB1THU = aux["SETTING_TB1THU"].as<AutoConnectCheckbox>().checked;
    preferences.putInt("TB1THU", SETTING_TB1THU);
    SETTING_TB1FRI = aux["SETTING_TB1FRI"].as<AutoConnectCheckbox>().checked;
    preferences.putInt("TB1FRI", SETTING_TB1FRI);
    SETTING_TB1SAT = aux["SETTING_TB1SAT"].as<AutoConnectCheckbox>().checked;
    preferences.putInt("TB1SAT", SETTING_TB1SAT);
    SETTING_TB1FROM = aux["SETTING_TB1FROM"].as<AutoConnectInput>().value.toInt();
    preferences.putInt("TB1FROM", SETTING_TB1FROM);
    SETTING_TB1TO = aux["SETTING_TB1TO"].as<AutoConnectInput>().value.toInt();
    preferences.putInt("TB1TO", SETTING_TB1TO);

    SETTING_TB2ENABLE = aux["SETTING_TB2ENABLE"].as<AutoConnectCheckbox>().checked;
    preferences.putInt("TB2ENABLE", SETTING_TB2ENABLE);
    SETTING_TB2BRIGHT = aux["SETTING_TB2BRIGHT"].as<AutoConnectInput>().value.toInt();
    preferences.putInt("TB2BRIGHT", SETTING_TB2BRIGHT);
    SETTING_TB2SUN = aux["SETTING_TB2SUN"].as<AutoConnectCheckbox>().checked;
    preferences.putInt("TB2SUN", SETTING_TB2SUN);
    SETTING_TB2MON = aux["SETTING_TB2MON"].as<AutoConnectCheckbox>().checked;
    preferences.putInt("TB2MON", SETTING_TB2MON);
    SETTING_TB2TUE = aux["SETTING_TB2TUE"].as<AutoConnectCheckbox>().checked;
    preferences.putInt("TB2TUE", SETTING_TB2TUE);
    SETTING_TB2WED = aux["SETTING_TB2WED"].as<AutoConnectCheckbox>().checked;
    preferences.putInt("TB2WED", SETTING_TB2WED);
    SETTING_TB2THU = aux["SETTING_TB2THU"].as<AutoConnectCheckbox>().checked;
    preferences.putInt("TB2THU", SETTING_TB2THU);
    SETTING_TB2FRI = aux["SETTING_TB2FRI"].as<AutoConnectCheckbox>().checked;
    preferences.putInt("TB2FRI", SETTING_TB2FRI);
    SETTING_TB2SAT = aux["SETTING_TB2SAT"].as<AutoConnectCheckbox>().checked;
    preferences.putInt("TB2SAT", SETTING_TB2SAT);
    SETTING_TB2FROM = aux["SETTING_TB2FROM"].as<AutoConnectInput>().value.toInt();
    preferences.putInt("TB2FROM", SETTING_TB2FROM);
    SETTING_TB2TO = aux["SETTING_TB2TO"].as<AutoConnectInput>().value.toInt();
    preferences.putInt("TB2TO", SETTING_TB2TO);

    SETTING_TB3ENABLE = aux["SETTING_TB3ENABLE"].as<AutoConnectCheckbox>().checked;
    preferences.putInt("TB3ENABLE", SETTING_TB3ENABLE);
    SETTING_TB3BRIGHT = aux["SETTING_TB3BRIGHT"].as<AutoConnectInput>().value.toInt();
    preferences.putInt("TB3BRIGHT", SETTING_TB3BRIGHT);
    SETTING_TB3SUN = aux["SETTING_TB3SUN"].as<AutoConnectCheckbox>().checked;
    preferences.putInt("TB3SUN", SETTING_TB3SUN);
    SETTING_TB3MON = aux["SETTING_TB3MON"].as<AutoConnectCheckbox>().checked;
    preferences.putInt("TB3MON", SETTING_TB3MON);
    SETTING_TB3TUE = aux["SETTING_TB3TUE"].as<AutoConnectCheckbox>().checked;
    preferences.putInt("TB3TUE", SETTING_TB3TUE);
    SETTING_TB3WED = aux["SETTING_TB3WED"].as<AutoConnectCheckbox>().checked;
    preferences.putInt("TB3WED", SETTING_TB3WED);
    SETTING_TB3THU = aux["SETTING_TB3THU"].as<AutoConnectCheckbox>().checked;
    preferences.putInt("TB3THU", SETTING_TB3THU);
    SETTING_TB3FRI = aux["SETTING_TB3FRI"].as<AutoConnectCheckbox>().checked;
    preferences.putInt("TB3FRI", SETTING_TB3FRI);
    SETTING_TB3SAT = aux["SETTING_TB3SAT"].as<AutoConnectCheckbox>().checked;
    preferences.putInt("TB3SAT", SETTING_TB3SAT);
    SETTING_TB3FROM = aux["SETTING_TB3FROM"].as<AutoConnectInput>().value.toInt();
    preferences.putInt("TB3FROM", SETTING_TB3FROM);
    SETTING_TB3TO = aux["SETTING_TB3TO"].as<AutoConnectInput>().value.toInt();
    preferences.putInt("TB3TO", SETTING_TB3TO);

    SETTING_TB4ENABLE = aux["SETTING_TB4ENABLE"].as<AutoConnectCheckbox>().checked;
    preferences.putInt("TB4ENABLE", SETTING_TB4ENABLE);
    SETTING_TB4BRIGHT = aux["SETTING_TB4BRIGHT"].as<AutoConnectInput>().value.toInt();
    preferences.putInt("TB4BRIGHT", SETTING_TB4BRIGHT);
    SETTING_TB4SUN = aux["SETTING_TB4SUN"].as<AutoConnectCheckbox>().checked;
    preferences.putInt("TB4SUN", SETTING_TB4SUN);
    SETTING_TB4MON = aux["SETTING_TB4MON"].as<AutoConnectCheckbox>().checked;
    preferences.putInt("TB4MON", SETTING_TB4MON);
    SETTING_TB4TUE = aux["SETTING_TB4TUE"].as<AutoConnectCheckbox>().checked;
    preferences.putInt("TB4TUE", SETTING_TB4TUE);
    SETTING_TB4WED = aux["SETTING_TB4WED"].as<AutoConnectCheckbox>().checked;
    preferences.putInt("TB4WED", SETTING_TB4WED);
    SETTING_TB4THU = aux["SETTING_TB4THU"].as<AutoConnectCheckbox>().checked;
    preferences.putInt("TB4THU", SETTING_TB4THU);
    SETTING_TB4FRI = aux["SETTING_TB4FRI"].as<AutoConnectCheckbox>().checked;
    preferences.putInt("TB4FRI", SETTING_TB4FRI);
    SETTING_TB4SAT = aux["SETTING_TB4SAT"].as<AutoConnectCheckbox>().checked;
    preferences.putInt("TB4SAT", SETTING_TB4SAT);
    SETTING_TB4FROM = aux["SETTING_TB4FROM"].as<AutoConnectInput>().value.toInt();
    preferences.putInt("TB4FROM", SETTING_TB4FROM);
    SETTING_TB4TO = aux["SETTING_TB4TO"].as<AutoConnectInput>().value.toInt();
    preferences.putInt("TB4TO", SETTING_TB4TO);
    
    preferences.end();
  }

  return String();
}

String onResults(AutoConnectAux& aux, PageArgument& args) {
  int valueA = args.arg("valueA").toInt();
  int valueB = args.arg("valueB").toInt();

  ledcWrite(0, valueA);

  aux["results"].as<AutoConnectText>().value = String(valueA) + " + " + String(valueB) + " = " + String(valueA + valueB);
  return String();
}
// END ADD PAGE TEST

void WRITE_BITS(int x) {
  #ifdef IV_9_6D_RR
  int var = 7;
  #else
  int var = 9;
  #endif
  
  while (var >= 0) {
    digitalWrite(CLOCK_PIN, 0);
    //digitalWrite(DATA_PIN, (x >> var) & 0x01);    // REGULAR
    digitalWrite(DATA_PIN, !((x >> var) & 0x01));      // INVERTED
    digitalWrite(CLOCK_PIN, 1);
    var--;
  }
}

void WRITE_ONELOWBIT() {
    digitalWrite(CLOCK_PIN, 0);
    digitalWrite(DATA_PIN, 1);
    digitalWrite(CLOCK_PIN, 1);
}

void WRITE_ONEHIGHBIT() {
    digitalWrite(CLOCK_PIN, 0);
    digitalWrite(DATA_PIN, 0);
    digitalWrite(CLOCK_PIN, 1);
}

#ifdef IV_9_6D_RR
byte BITS_FOR_THE_UPSIDE_DOWN(byte TOP) // THIS IS FOR IV-9 clock where tubes are mounted on the bottom of the PCB
{
  byte BOTTOM = 0;

  bitWrite(BOTTOM, 0, bitRead(TOP,0));
  bitWrite(BOTTOM, 1, bitRead(TOP,7));
  bitWrite(BOTTOM, 2, bitRead(TOP,6));
  bitWrite(BOTTOM, 3, bitRead(TOP,5));
  bitWrite(BOTTOM, 4, bitRead(TOP,4));
  bitWrite(BOTTOM, 5, bitRead(TOP,3));
  bitWrite(BOTTOM, 6, bitRead(TOP,2));
  bitWrite(BOTTOM, 7, bitRead(TOP,1));

  return BOTTOM;
}
#endif

void WRITE_DISPLAY_VAR(int HT, int HO, int MT, int MO, int ST, int SO, byte HT_DP, byte HO_DP, byte MT_DP, byte MO_DP, byte ST_DP, byte SO_DP) {
//  prev_sec_ones = sec_ones;
//  prev_sec_tens = sec_tens;
//  prev_min_ones = min_ones;
//  prev_min_tens = min_tens;
//  prev_hour_ones = hour_ones;
//  prev_hour_tens = hour_tens;

  prev_sec_ones = SO;
  prev_sec_tens = ST;
  prev_min_ones = MO;
  prev_min_tens = MT;
  prev_hour_ones = HO;
  prev_hour_tens = HT;

  sec_ones = SO;
  sec_tens = ST;
  min_ones = MO;
  min_tens = MT;
  hour_ones = HO;
  hour_tens = HT;

  prev_hour_tens_dp = HT_DP; prev_hour_ones_dp = HO_DP; prev_min_tens_dp = MT_DP; prev_min_ones_dp = MO_DP; prev_sec_tens_dp = ST_DP; prev_sec_ones_dp = SO_DP;

  hour_tens_dp = HT_DP; hour_ones_dp = HO_DP; min_tens_dp = MT_DP; min_ones_dp = MO_DP; sec_tens_dp = ST_DP; sec_ones_dp = SO_DP;
}

#ifdef IV_9_6D_RR
void WRITE_DISPLAY_IV9(int HT, int HO, int MT, int MO, int ST, int SO, byte HT_DP, byte HO_DP, byte MT_DP, byte MO_DP, byte ST_DP, byte SO_DP)
{

  if ( SETTING_TUBEONBOTTOM == 2 ) {  // THIS IS FOR IV-9 clock where tubes are mounted on the bottom of the PCB
    WRITE_BITS((SO_DP ^ BITS_FOR_THE_UPSIDE_DOWN(IV9_DIGIT_MAP[SO])));
    WRITE_BITS((ST_DP ^ BITS_FOR_THE_UPSIDE_DOWN(IV9_DIGIT_MAP[ST])));
    WRITE_BITS((MO_DP ^ BITS_FOR_THE_UPSIDE_DOWN(IV9_DIGIT_MAP[MO])));
    WRITE_BITS((MT_DP ^ BITS_FOR_THE_UPSIDE_DOWN(IV9_DIGIT_MAP[MT])));
    WRITE_BITS((HO_DP ^ BITS_FOR_THE_UPSIDE_DOWN(IV9_DIGIT_MAP[HO])));
    WRITE_BITS((HT_DP ^ BITS_FOR_THE_UPSIDE_DOWN(IV9_DIGIT_MAP[HT])));    
  } else {
    WRITE_BITS((HT_DP ^ IV9_DIGIT_MAP[HT]));
    WRITE_BITS((HO_DP ^ IV9_DIGIT_MAP[HO]));
    WRITE_BITS((MT_DP ^ IV9_DIGIT_MAP[MT]));
    WRITE_BITS((MO_DP ^ IV9_DIGIT_MAP[MO]));
    WRITE_BITS((ST_DP ^ IV9_DIGIT_MAP[ST]));
    WRITE_BITS((SO_DP ^ IV9_DIGIT_MAP[SO]));
  }
     
  digitalWrite(RCK_PIN, 1);
  digitalWrite(RCK_PIN, 0);
}
#endif

#ifdef Z57XM6DV2
void WRITE_DISPLAY_Z57(int HT, int HO, int MT, int MO, int ST, int SO, byte HT_DP, byte HO_DP, byte MT_DP, byte MO_DP, byte ST_DP, byte SO_DP)
{
    // THIS CHECK ADDED IN v0.92 BETA2 for the Z57 NIXIE CLOCK ONLY. This "if" block makes sure that a digit is changing before sending out the data via the shift registers.
    // This was needed because there are no storage resiters on the driver IC's that are driving the tubes, this results in SLIGHT ghosting of tubes which are BLANKED when
    // we update the display at higher rates (normally 300hz+). Although minor and not very noticable, checking if the display digits have changed
    // before updating the actual shift register avoids this issue. We do need the high refresh rate to be able to cross fade the digits smoothly which was added in v0.92 BETA2
    if ( display_sec_ones == SO && display_sec_tens == ST && display_min_ones == MO && display_min_tens == MT && display_hour_ones == HO && display_hour_tens == HT
      && display_sec_ones_dp == SO_DP && display_sec_tens_dp == ST_DP && display_min_ones_dp == MO_DP && display_min_tens_dp == MT_DP && display_hour_ones_dp == HO_DP && display_hour_tens_dp == HT_DP ) {
      return;
    }
    display_sec_ones=SO; display_sec_tens=ST; display_min_ones=MO; display_min_tens=MT; display_hour_ones=HO; display_hour_tens=HT;
    display_sec_ones_dp=SO_DP; display_sec_tens_dp=ST_DP; display_min_ones_dp=MO_DP; display_min_tens_dp=MT_DP; display_hour_ones_dp=HO_DP; display_hour_tens_dp=HT_DP;
    
    WRITE_BITS(MIN_TEN_MAP[MT]);

    if ( HO_DP == 1 ) { // DECIMAL POINT ON
      WRITE_ONEHIGHBIT();    // NEON BULB
    } else {
      WRITE_ONELOWBIT();    // NEON BULB
    }
    
    WRITE_BITS(HOUR_ONE_MAP[HO]);
    WRITE_BITS(HOUR_TEN_MAP[HT]);
    
    WRITE_ONELOWBIT();    // PIN 35 U4 /  NO CONNECTION

    WRITE_BITS(SEC_ONE_MAP[SO]);
    WRITE_BITS(SEC_TEN_MAP[ST]);
    
    if ( MO_DP == 1 ) { // DECIMAL POINT ON
      WRITE_ONEHIGHBIT();    // NEON BULB
    } else {
      WRITE_ONELOWBIT();    // NEON BULB
    }
    
    WRITE_BITS(MIN_ONE_MAP[MO]);
    
    WRITE_ONELOWBIT();    // PIN 35 NO CONNECTION  

    digitalWrite(CLOCK_PIN, 0);   // WRITE CLOCK PIN TO LOW TO KEEP LOGIC LEVEL SHIFTER (RESISTOR AND FET) TEMPERATURES LOWER
    digitalWrite(DATA_PIN, 0);    // WRITE DATA PIN TO LOW TO KEEP LOGIC LEVEL SHIFTER (RESISTOR AND FET) TEMPERATURES LOWER
}
#endif

void demo_digits(){
    int var = 0;
    #ifdef IV_9_6D_RR
      while (var <= 9) { // SET TO 37 TO DISPLAY THE WHOLE ALPHABET!
    #else 
      while (var <= 9) {
    #endif
      WRITE_DISPLAY_VAR(var, var, var, var, var, var, 0, 0, 0, 0, 0, 0);
      //hour_tens = var; hour_ones = var; min_tens = var; min_ones = var; sec_tens = var; sec_ones = var;
      var++;
      delay(500);      
    }
  //WRITE_DISPLAY(9,9,9,9,9,9, 0, 0, 0, 0, 0, 0);   
}

void setTimezone(String timezone){
  serial_debug_printf(" Setting Timezone to %s\n",timezone.c_str());
  setenv("TZ",timezone.c_str(),1);  //  Now adjust the TZ.  Clock settings are adjusted to show the new local time
  tzset();
}

void initTime(String timezone){
  //struct tm timeinfo; // Moved to global

  serial_debugln("Setting up time");
  //configTime(0, 0, "pool.ntp.org");    // First connect to NTP server, with 0 TZ offset
  configTime(0, 0, SETTING_NTPSERVER.c_str());    // First connect to NTP server, with 0 TZ offset
  if(!getLocalTime(&timeinfo)){
    serial_debugln(" Failed to obtain time");
    //return; // REMOVED AS OF 0.91 BETA4, this was at times causing timezone to not set on initial bootup
  } else {
    serial_debugln(" Got the time from NTP");
  }
  
  setTimezone(timezone);
}

void printLocalTime(){
  //struct tm timeinfo; // Moved to global
  if(!getLocalTime(&timeinfo)){
    serial_debugln("Failed to obtain time 1");
    return;
  }
  serial_debug_x2(&timeinfo, "%A, %B %d %Y %H:%M:%S zone %Z %z ");
}

void setTime(int yr, int month, int mday, int hr, int minute, int sec, int isDst){
  struct tm tm;

  tm.tm_year = yr - 1900;   // Set date
  tm.tm_mon = month-1;
  tm.tm_mday = mday;
  tm.tm_hour = hr;      // Set time
  tm.tm_min = minute;
  tm.tm_sec = sec;
  tm.tm_isdst = isDst;  // 1 or 0
  time_t t = mktime(&tm);
  serial_debug_printf("Setting time: %s", asctime(&tm));
  struct timeval now = { .tv_sec = t };
  settimeofday(&now, NULL);
}

#ifdef Z57XM6DV2
void cathodepp (void)
{
  if ( (timeinfo.tm_hour >= SETTING_CATHODEPPSTART && timeinfo.tm_hour < (SETTING_CATHODEPPSTART+SETTING_CATHODEPPDUR)) || timeinfo.tm_hour < ((SETTING_CATHODEPPSTART+SETTING_CATHODEPPDUR)-24) ) {
    if ( SETTING_CATHODEPP == 3 ) {
       //ledcWrite(0, 0);     // SET TO FULL BRIGHTNESS
       BRIGHTNESS=255;
    }
    if ( SETTING_CATHODEPP == 4 ) {
       //ledcWrite(0, 128);   // SET TO HALF BRIGHTNESS
       BRIGHTNESS=128;
    }

    //if ( timeinfo.tm_sec == 0 || timeinfo.tm_sec == 10 || timeinfo.tm_sec == 20 || timeinfo.tm_sec == 30 || timeinfo.tm_sec == 40 || timeinfo.tm_sec == 50 ) { // OLD REMOVED 0.91 BETA4
    if ( timeinfo.tm_sec%SETTING_CATHODEPPREV < SETTING_CATHODEPPREVD ) {   // NEW 0.91 BETA4 Check if we need to revert to a regular display every SETTING_CATHODEPPREV (SECONDS) for SETTING_CATHODEPPREVD (DURATION)
      //juggleoutput(0);
    } else {
      DISPLAY_OTHER_THEN_TIME=1;
      sec_ones=timeinfo.tm_min/6;
      sec_tens=timeinfo.tm_min/6;
      min_ones=timeinfo.tm_min/6;
      min_tens=timeinfo.tm_min/6;
      hour_ones=timeinfo.tm_min/6;
      hour_tens=timeinfo.tm_min/6;
    }
              
  }
}
#endif

#ifdef BMP_ENABLE
void read_bmp() {
  if ( SETTING_BMP == 2 || (SETTING_BMP == 3 && timeinfo.tm_sec%10 == 0) ) {  // 2 is read every second (function should be in 'every second' loop, and 3 is every 10 seconds
    bmp.begin(0x77);
    // READ BMP280 PRESSURE/ALTITUDE/TEMPERATURE
    //Serial.print(F("Temperature = ")); Serial.print(bmp.readTemperature()); Serial.println(" *C");
    //Serial.print(F("Pressure = ")); Serial.print(bmp.readPressure()); Serial.println(" Pa");
    //Serial.print(F("Approx altitude = ")); Serial.print(bmp.readAltitude(1013.25)); /* Adjusted to local forecast! */ Serial.println(" m");
    bmp_PA = bmp.readPressure();
    bmp_METERS = bmp.readAltitude(1013.25); // STANDARD
    bmp_TEMPERATURE_C = bmp.readTemperature();
    bmp_HG = bmp_PA * 0.00029530;
    bmp_FEET = bmp_METERS * 3.2808;
  }
}
#endif

void display_temperature() {
  
  if ( SETTING_TEMPDISP_SEC != 0 ) {
    if ( SETTING_TEMPHUM == 2 ) { // READY EVERY SECOND
      //aht.begin();  // THIS IS MOVED OUT OF THE REGULAR LOOP
      sensors_event_t aht_humidity, aht_temp;
      aht.getEvent(&aht_humidity, &aht_temp);// populate temp and humidity objects with fresh data
      HUMIDITY=aht_humidity.relative_humidity + SETTING_HUMIDITY_COMP;
      TEMPERATURE_C=aht_temp.temperature + (SETTING_TEMP_COMP/float(10));
      TEMPERATURE_F=(TEMPERATURE_C * 9/5) + 32;
    } else if ( timeinfo.tm_sec%10 == 0 ) {    // READ TEMP AND HUMIDITY EVERY 10 SECONDS
      //aht.begin();  // THIS IS MOVED OUT OF THE REGULAR LOOP
      sensors_event_t aht_humidity, aht_temp;
      aht.getEvent(&aht_humidity, &aht_temp);// populate temp and humidity objects with fresh data
      HUMIDITY=aht_humidity.relative_humidity + SETTING_HUMIDITY_COMP;
      TEMPERATURE_C=aht_temp.temperature + (SETTING_TEMP_COMP/float(10));
      TEMPERATURE_F=(TEMPERATURE_C * 9/5) + 32;
    }
    //if ( (timeinfo.tm_sec%SETTING_TEMPDISP_SEC <= SETTING_TEMPDISP_DUR) || ((timeinfo.tm_sec%SETTING_TEMPDISP_SEC)-1) == 0 ) {
    if ( timeinfo.tm_sec%SETTING_TEMPDISP_SEC < SETTING_TEMPDISP_DUR ) {
      if ( SETTING_TEMP_IN_C == 1 ) {
          DISPLAY_OTHER_THEN_TIME=1;
          hour_tens=int(TEMPERATURE_C) / 10;
          hour_ones=int(TEMPERATURE_C) % 10;;
          min_tens=int(TEMPERATURE_C * 10) %10;
          min_ones=DIGIT_BLANK;      
          sec_tens=int(HUMIDITY) / 10;
          sec_ones=int(HUMIDITY) % 10;
          hour_tens_dp = 0; hour_ones_dp = 1; min_tens_dp = 0; min_ones_dp = 0; sec_tens_dp = 0; sec_ones_dp = 0;
      } else {
        if ( TEMPERATURE_F >= 100 ) {          // 100+F display as xxx and round
          DISPLAY_OTHER_THEN_TIME=1;
          hour_tens=int(round(TEMPERATURE_F)) %1000/100;
          hour_ones=int(round(TEMPERATURE_F)) %100/10;
          min_tens=int(round(TEMPERATURE_F)) % 10;
          min_ones=DIGIT_BLANK;
          sec_tens=int(HUMIDITY) / 10;
          sec_ones=int(HUMIDITY) % 10;
          hour_tens_dp = 0; hour_ones_dp = 0; min_tens_dp = 0; min_ones_dp = 0; sec_tens_dp = 0; sec_ones_dp = 0;
        } else {                              // LESS THEN 100F display as xx.x  
          DISPLAY_OTHER_THEN_TIME=1;
          hour_tens=int(TEMPERATURE_F) / 10;
          hour_ones=int(TEMPERATURE_F) % 10;;
          min_tens=int(TEMPERATURE_F * 10) %10;
          min_ones=DIGIT_BLANK;      
          sec_tens=int(HUMIDITY) / 10;
          sec_ones=int(HUMIDITY) % 10;
          hour_tens_dp = 0; hour_ones_dp = 1; min_tens_dp = 0; min_ones_dp = 0; sec_tens_dp = 0; sec_ones_dp = 0;
        }
      }
    }
    //Serial.print("Temp: "); Serial.print(TEMPERATURE_C); Serial.print(" C "); Serial.print(TEMPERATURE_F); Serial.print(" F "); Serial.print("Humidity: "); Serial.print(HUMIDITY); Serial.print("% rH ");
    serial_debug("Temp: "); serial_debug(TEMPERATURE_C); serial_debug(" C "); serial_debug(TEMPERATURE_F); serial_debug(" F "); serial_debug("Humidity: "); serial_debug(HUMIDITY); serial_debug("% rH ");
  } else {  // TEMPERATURE DISPLAY IS NOT ON, SO WE CHECK IF WE NEED TO READ THE SENSOR ANYWAY (FOR WEB DISPLAY PURPOSES ETC)
    if ( SETTING_TEMPHUM == 2 ) { // READY EVERY SECOND
      sensors_event_t aht_humidity, aht_temp;
      aht.getEvent(&aht_humidity, &aht_temp);// populate temp and humidity objects with fresh data
      HUMIDITY=aht_humidity.relative_humidity + SETTING_HUMIDITY_COMP;
      TEMPERATURE_C=aht_temp.temperature + (SETTING_TEMP_COMP/float(10));
      TEMPERATURE_F=(TEMPERATURE_C * 9/5) + 32;
    } else if ( SETTING_TEMPHUM == 3 && timeinfo.tm_sec%10 == 0 ) { // READ EVERY 10 SECONDS
      sensors_event_t aht_humidity, aht_temp;
      aht.getEvent(&aht_humidity, &aht_temp);// populate temp and humidity objects with fresh data
      HUMIDITY=aht_humidity.relative_humidity + SETTING_HUMIDITY_COMP;
      TEMPERATURE_C=aht_temp.temperature + (SETTING_TEMP_COMP/float(10));
      TEMPERATURE_F=(TEMPERATURE_C * 9/5) + 32;
    }
  }
}

void display_date() {
  if ( SETTING_DATEDISP_SEC != 0 ) {
    if ( timeinfo.tm_sec%SETTING_DATEDISP_SEC < SETTING_DATEDISP_DUR ) {
      switch (SETTING_DATEFORMAT) {
         case 2:
          DISPLAY_OTHER_THEN_TIME=1;
          sec_ones = timeinfo.tm_mday % 10;
          sec_tens = timeinfo.tm_mday / 10;
          min_ones = (timeinfo.tm_mon+1) % 10;
          min_tens = (timeinfo.tm_mon+1) / 10;
          hour_ones = (timeinfo.tm_year-100) % 10;
          hour_tens = (timeinfo.tm_year-100) / 10;
          break;
         case 3:
          DISPLAY_OTHER_THEN_TIME=1;
          sec_ones = (timeinfo.tm_year-100) % 10;
          sec_tens = (timeinfo.tm_year-100) / 10;
          min_ones = (timeinfo.tm_mon+1) % 10;
          min_tens = (timeinfo.tm_mon+1) / 10;
          hour_ones = timeinfo.tm_mday % 10;
          hour_tens = timeinfo.tm_mday / 10;
          break;          
         default: // THIS IS CASE 1, our default case
          DISPLAY_OTHER_THEN_TIME=1;
          sec_ones = (timeinfo.tm_year-100) % 10;
          sec_tens = (timeinfo.tm_year-100) / 10;
          min_ones = timeinfo.tm_mday % 10;
          min_tens = timeinfo.tm_mday / 10;
          hour_ones = (timeinfo.tm_mon+1) % 10;
          hour_tens = (timeinfo.tm_mon+1) / 10;
          break;
      }
    }
  }
}

void GET_ESP_PARITIONS() {
  char PARTOUTPUT[500];
  //ESP_LOGI(TAG, "Start test");
  esp_partition_iterator_t iter = esp_partition_find(ESP_PARTITION_TYPE_APP, ESP_PARTITION_SUBTYPE_ANY, NULL);
  //ESP_LOGI(TAG, "Name, type, subtype, offset, length");
  snprintf(PARTOUTPUT, sizeof(PARTOUTPUT), "Name, type, subtype, offset, length");
  while (iter != nullptr)
  {
    const esp_partition_t *partition = esp_partition_get(iter);
    //ESP_LOGI(TAG, "%s, app, %d, 0x%x, 0x%x (%d)", partition->label, partition->subtype, partition->address, partition->size, partition->size);
    snprintf(PARTOUTPUT, sizeof(PARTOUTPUT), "%s<br>%s, app, %d, 0x%x, 0x%x (%d)", PARTOUTPUT, partition->label, partition->subtype, partition->address, partition->size, partition->size);
    iter = esp_partition_next(iter);
  }
  
  esp_partition_iterator_release(iter);
  iter = esp_partition_find(ESP_PARTITION_TYPE_DATA, ESP_PARTITION_SUBTYPE_ANY, NULL);
  while (iter != nullptr)
  {
    const esp_partition_t *partition = esp_partition_get(iter);
    //ESP_LOGI(TAG, "%s, data, %d, 0x%x, 0x%x (%d)", partition->label, partition->subtype, partition->address, partition->size, partition->size);
    snprintf(PARTOUTPUT, sizeof(PARTOUTPUT), "%s<br>%s, data, %d, 0x%x, 0x%x (%d)", PARTOUTPUT, partition->label, partition->subtype, partition->address, partition->size, partition->size);
    iter = esp_partition_next(iter);
  }    
  esp_partition_iterator_release(iter);
  
  Server.send(200, "text/html", PARTOUTPUT);
}

void mqttcallback(char* topic, byte* message, unsigned int length) {
  String messagetemp;
  
  for (int i = 0; i < length; i++) {
    messagetemp += (char)message[i];
  }

  if ( SETTING_MQTTBRIGHT == 1 ) { 
    //if (String(topic) == "clock/neonixie/iv9clock-bd98/subscribe_brightness") {
    if (String(topic) == mqtt_topic_subscribe_brightness) {
      SETTING_BRIGHTNESS=messagetemp.toInt();
    }
  }
}

void alarm_process() {
  //int makenoise = 0;
  makenoise = 0;
  int alarm1_duration_check;
  int alarm2_duration_check;
  int alarm3_duration_check;
  
  int alarm_ONETIMEPLAY_duration_check;

  // START ONCE PER MINUTE LOOP
  if ( ALARM_PREV_MIN != timeinfo.tm_min ) { 
    ALARM_PREV_MIN=timeinfo.tm_min;

    if ( SETTING_ALARM1ENABLE == 1 ) {
      if (  (SETTING_ALARM1SUN == 1 && timeinfo.tm_wday == 0) ||
            (SETTING_ALARM1MON == 1 && timeinfo.tm_wday == 1) ||
            (SETTING_ALARM1TUE == 1 && timeinfo.tm_wday == 2) ||
            (SETTING_ALARM1WED == 1 && timeinfo.tm_wday == 3) ||
            (SETTING_ALARM1THU == 1 && timeinfo.tm_wday == 4) ||
            (SETTING_ALARM1FRI == 1 && timeinfo.tm_wday == 5) ||
            (SETTING_ALARM1SAT == 1 && timeinfo.tm_wday == 6) ) {
        if ( ( timeinfo.tm_hour == SETTING_ALARM1HOUR || SETTING_ALARM1HOUR == 24 ) && ( timeinfo.tm_min == SETTING_ALARM1MIN || SETTING_ALARM1MIN == 60 ) ) {
          ALARM1_DURATION = 0;            // SET ALARM DURATION TO 0 SO WE CAN START ALARMING
        }
      }
      if ( SETTING_ALARM1SUN == 0 && SETTING_ALARM1MON == 0 && SETTING_ALARM1TUE == 0 && SETTING_ALARM1WED == 0 && SETTING_ALARM1THU == 0 && SETTING_ALARM1FRI == 0 && SETTING_ALARM1SAT == 0 ) {
        if ( ( SETTING_ALARM1MONTH == (timeinfo.tm_mon+1) || SETTING_ALARM1MONTH == 13 ) && ( SETTING_ALARM1DAY == (timeinfo.tm_mday) || SETTING_ALARM1DAY == 32 ) ) {
          if ( ( timeinfo.tm_hour == SETTING_ALARM1HOUR || SETTING_ALARM1HOUR == 24 ) && ( timeinfo.tm_min == SETTING_ALARM1MIN || SETTING_ALARM1MIN == 60 ) ) {
            ALARM1_DURATION = 0;            // SET ALARM DURATION TO 0 SO WE CAN START ALARMING
          }
        }        
      }
    }
    
    if ( SETTING_ALARM2ENABLE == 1 ) {
      if (  (SETTING_ALARM2SUN == 1 && timeinfo.tm_wday == 0) ||
            (SETTING_ALARM2MON == 1 && timeinfo.tm_wday == 1) ||
            (SETTING_ALARM2TUE == 1 && timeinfo.tm_wday == 2) ||
            (SETTING_ALARM2WED == 1 && timeinfo.tm_wday == 3) ||
            (SETTING_ALARM2THU == 1 && timeinfo.tm_wday == 4) ||
            (SETTING_ALARM2FRI == 1 && timeinfo.tm_wday == 5) ||
            (SETTING_ALARM2SAT == 1 && timeinfo.tm_wday == 6) ) {
        if ( ( timeinfo.tm_hour == SETTING_ALARM2HOUR || SETTING_ALARM2HOUR == 24 ) && ( timeinfo.tm_min == SETTING_ALARM2MIN || SETTING_ALARM2MIN == 60 ) ) {
          ALARM2_DURATION = 0;            // SET ALARM DURATION TO 0 SO WE CAN START ALARMING
        }
      }
      if ( SETTING_ALARM2SUN == 0 && SETTING_ALARM2MON == 0 && SETTING_ALARM2TUE == 0 && SETTING_ALARM2WED == 0 && SETTING_ALARM2THU == 0 && SETTING_ALARM2FRI == 0 && SETTING_ALARM2SAT == 0 ) {
        if ( ( SETTING_ALARM2MONTH == (timeinfo.tm_mon+1) || SETTING_ALARM2MONTH == 13 ) && ( SETTING_ALARM2DAY == (timeinfo.tm_mday) || SETTING_ALARM2DAY == 32 ) ) {
          if ( ( timeinfo.tm_hour == SETTING_ALARM2HOUR || SETTING_ALARM2HOUR == 24 ) && ( timeinfo.tm_min == SETTING_ALARM2MIN || SETTING_ALARM2MIN == 60 ) ) {
            ALARM2_DURATION = 0;            // SET ALARM DURATION TO 0 SO WE CAN START ALARMING
          }
        }        
      }
    }
        
    if ( SETTING_ALARM3ENABLE == 1 ) {
      if (  (SETTING_ALARM3SUN == 1 && timeinfo.tm_wday == 0) ||
            (SETTING_ALARM3MON == 1 && timeinfo.tm_wday == 1) ||
            (SETTING_ALARM3TUE == 1 && timeinfo.tm_wday == 2) ||
            (SETTING_ALARM3WED == 1 && timeinfo.tm_wday == 3) ||
            (SETTING_ALARM3THU == 1 && timeinfo.tm_wday == 4) ||
            (SETTING_ALARM3FRI == 1 && timeinfo.tm_wday == 5) ||
            (SETTING_ALARM3SAT == 1 && timeinfo.tm_wday == 6) ) {
        if ( ( timeinfo.tm_hour == SETTING_ALARM3HOUR || SETTING_ALARM3HOUR == 24 ) && ( timeinfo.tm_min == SETTING_ALARM3MIN || SETTING_ALARM3MIN == 60 ) ) {
          ALARM3_DURATION = 0;            // SET ALARM DURATION TO 0 SO WE CAN START ALARMING
        }
      }
      if ( SETTING_ALARM3SUN == 0 && SETTING_ALARM3MON == 0 && SETTING_ALARM3TUE == 0 && SETTING_ALARM3WED == 0 && SETTING_ALARM3THU == 0 && SETTING_ALARM3FRI == 0 && SETTING_ALARM3SAT == 0 ) {
        if ( ( SETTING_ALARM3MONTH == (timeinfo.tm_mon+1) || SETTING_ALARM3MONTH == 13 ) && ( SETTING_ALARM3DAY == (timeinfo.tm_mday) || SETTING_ALARM3DAY == 32 ) ) {
          if ( ( timeinfo.tm_hour == SETTING_ALARM3HOUR || SETTING_ALARM3HOUR == 24 ) && ( timeinfo.tm_min == SETTING_ALARM3MIN || SETTING_ALARM3MIN == 60 ) ) {
            ALARM3_DURATION = 0;            // SET ALARM DURATION TO 0 SO WE CAN START ALARMING
          }
        }        
      }
    }    
  
  } // END ONCE PER MINUTE LOOP

  if ( SETTING_ALARM1DUR == 1 ) {   // FIND OUT THE DURATION OF THE ALARM IN SECONDS
    alarm1_duration_check = 87000;  // Until canceled
  } else if ( SETTING_ALARM1DUR == 2 ) {
    alarm1_duration_check = 2;  // 2 seconds
  } else if ( SETTING_ALARM1DUR == 3 ) {
    alarm1_duration_check = 5;  // 5 seconds
  } else if ( SETTING_ALARM1DUR == 4 ) {
    alarm1_duration_check = 10;  // 10 seconds
  } else if ( SETTING_ALARM1DUR == 5 ) {
    alarm1_duration_check = 30;  // 30 seconds
  } else {
    alarm1_duration_check = (SETTING_ALARM1DUR - 5) * 60; // Otherwise subtract 5 and we get the minutes which we convert to seconds
  }

  if ( SETTING_ALARM2DUR == 1 ) {   // FIND OUT THE DURATION OF THE ALARM IN SECONDS
    alarm2_duration_check = 87000;  // Until canceled
  } else if ( SETTING_ALARM2DUR == 2 ) {
    alarm2_duration_check = 2;  // 2 seconds
  } else if ( SETTING_ALARM2DUR == 3 ) {
    alarm2_duration_check = 5;  // 5 seconds
  } else if ( SETTING_ALARM2DUR == 4 ) {
    alarm2_duration_check = 10;  // 10 seconds
  } else if ( SETTING_ALARM2DUR == 5 ) {
    alarm2_duration_check = 30;  // 30 seconds
  } else {
    alarm2_duration_check = (SETTING_ALARM2DUR - 5) * 60; // Otherwise subtract 5 and we get the minutes which we convert to seconds
  }

  if ( SETTING_ALARM3DUR == 1 ) {   // FIND OUT THE DURATION OF THE ALARM IN SECONDS
    alarm3_duration_check = 87000;  // Until canceled
  } else if ( SETTING_ALARM3DUR == 2 ) {
    alarm3_duration_check = 2;  // 2 seconds
  } else if ( SETTING_ALARM3DUR == 3 ) {
    alarm3_duration_check = 5;  // 5 seconds
  } else if ( SETTING_ALARM3DUR == 4 ) {
    alarm3_duration_check = 10;  // 10 seconds
  } else if ( SETTING_ALARM3DUR == 5 ) {
    alarm3_duration_check = 30;  // 30 seconds
  } else {
    alarm3_duration_check = (SETTING_ALARM3DUR - 5) * 60; // Otherwise subtract 5 and we get the minutes which we convert to seconds
  }


  if ( SETTING_ONETIMEPLAY != 1 ) {
    makenoise = SETTING_ONETIMEPLAY-1;  
  }
  
  if ( ALARM1_DURATION < alarm1_duration_check ) {
    makenoise = SETTING_ALARM1TUNE;
    ALARM1_DURATION++;
  }

  if ( ALARM2_DURATION < alarm2_duration_check ) {
    makenoise = SETTING_ALARM2TUNE;
    ALARM2_DURATION++;
  }

  if ( ALARM3_DURATION < alarm3_duration_check ) {
    makenoise = SETTING_ALARM3TUNE;
    ALARM3_DURATION++;
  }


//  // THIS IS THE WORKING ONE
//  if ( makenoise == 1 ) {
//    if ( timeinfo.tm_sec % 2 == 0 ) {
//      ledcWriteNote(2, NOTE_E, 4);  
//    } else {
//      ledcWriteNote(2, NOTE_C, 4);   
//    }
//  } else {
//    ledcWriteTone(2, 0);
//  }


  // FOR REFERENCE, MOVED FROM MAIN LOOP
  
  // https://diyi0t.com/active-passive-buzzer-arduino-esp8266-esp32/
  // https://espressif-docs.readthedocs-hosted.com/projects/arduino-esp32/en/latest/api/ledc.html
  //tone(BUZZER_PIN, timeinfo.tm_sec*100);
  //ledcWriteTone(2, timeinfo.tm_sec*100);

    // DOORBELL LIKE
//  if ( timeinfo.tm_sec % 10 == 0 ) {
//    ledcWriteNote(2, NOTE_E, 4);  
//    delay(500);
//    ledcWriteNote(2, NOTE_C, 4);
//    delay(500);
//    ledcWriteTone(2, 0);  
//  }

//  unsigned long NOTES_currentMillis = millis();
//  if ( NOTES_currentMillis - NOTES_previousMillis >= NOTES_interval ) {
//    if ( timeinfo.tm_sec % 2 == 0 ) {
//      ledcWriteNote(2, NOTE_E, 4);  
//    } else {
//      ledcWriteNote(2, NOTE_C, 4);   
//    }
//    delay(10);
//    ledcWriteTone(2, 0);  
//    NOTES_previousMillis = NOTES_currentMillis;
//  }

  
}

void make_noise_check () {
  if ( makenoise != 0 ) {
    digitalWrite(ALARM_PIN, 1);
    if ( !anyrtttl::nonblocking::isPlaying() ) {
      switch (makenoise) {
        case 1:
          anyrtttl::nonblocking::begin(BUZZER_PIN, SETTING_ALARMNOTE1); 
          break;
        case 2:
          anyrtttl::nonblocking::begin(BUZZER_PIN, SETTING_ALARMNOTE2);
          break;
        case 3:
          anyrtttl::nonblocking::begin(BUZZER_PIN, SETTING_ALARMNOTE3);
          break;
        case 4:
          anyrtttl::nonblocking::begin(BUZZER_PIN, SETTING_ALARMNOTE4);
          break;
        case 5:
          anyrtttl::nonblocking::begin(BUZZER_PIN, SETTING_ALARMNOTE5);
          break;
        case 6:
          anyrtttl::nonblocking::begin(BUZZER_PIN, SETTING_ALARMNOTE6);
          break;
        case 7:
          anyrtttl::nonblocking::begin(BUZZER_PIN, SETTING_ALARMNOTE7);
          break;
        case 8:
          anyrtttl::nonblocking::begin(BUZZER_PIN, SETTING_RTTTL1.c_str());
          break;
        case 9:
          anyrtttl::nonblocking::begin(BUZZER_PIN, SETTING_RTTTL2.c_str());
          break;
        case 10:
          anyrtttl::nonblocking::begin(BUZZER_PIN, SETTING_RTTTL3.c_str());
          break;
      }      
    } else {
      anyrtttl::nonblocking::play();
    }
  } else {
    digitalWrite(ALARM_PIN, 0);
    if ( anyrtttl::nonblocking::isPlaying() ) {
      anyrtttl::nonblocking::stop();
    }
    pinMode(BUZZER_PIN, INPUT);   // WITHOUT CHANGING THIS PIN BACK TO INPUT, WE GET STRANGE BUZZING NOISES AFTER PLAYBACK, POSSIBLE BUG  
  }

}

void timed_brightness_process() {
  int hourmin = (timeinfo.tm_hour * 100) + timeinfo.tm_min; // The time now in a format HHMM to compare to our settings

  if ( SETTING_TB4ENABLE == 1 ) {
    if ( hourmin >= SETTING_TB4FROM && hourmin < SETTING_TB4TO ) {
      BRIGHTNESS=SETTING_TB4BRIGHT;
    }
  }

  if ( SETTING_TB3ENABLE == 1 ) {
    if ( hourmin >= SETTING_TB3FROM && hourmin < SETTING_TB3TO ) {
      BRIGHTNESS=SETTING_TB3BRIGHT;
    }
  }

  if ( SETTING_TB2ENABLE == 1 ) {
    if ( hourmin >= SETTING_TB2FROM && hourmin < SETTING_TB2TO ) {
      BRIGHTNESS=SETTING_TB2BRIGHT;
    }
  }

  if ( SETTING_TB1ENABLE == 1 ) {
    if ( hourmin >= SETTING_TB1FROM && hourmin < SETTING_TB1TO ) {
      BRIGHTNESS=SETTING_TB1BRIGHT;
    }
  }
  
}

int TIMERBLINK=0;
hw_timer_t *My_timer = NULL;
void IRAM_ATTR onTimer(){

  int SECONDS_FRACTION_COUNTER_divisor = 18;    // 18 works good for 1600hz IV9
  int SECONDS_FRACTION_COUNTER_divide= SECONDS_FRACTION_COUNTER / SECONDS_FRACTION_COUNTER_divisor;
  int SECONDS_FRACTION_COUNTER_remain= SECONDS_FRACTION_COUNTER % SECONDS_FRACTION_COUNTER_divisor;

#ifdef IV_9_6D_RR
  if ( SETTING_KEEPWARMRATIO != 0 ) {     // KEEPING THE UNLIT SEGMENTS WARM
    if ( TIMERBLINK < (SETTING_KEEPWARMRATIO-1) ) {
      if ( SETTING_CROSSFADE == 1 ) {
        if ( SECONDS_FRACTION_COUNTER_remain < ((SECONDS_FRACTION_COUNTER_divisor) - SECONDS_FRACTION_COUNTER_divide) ) {
          WRITE_DISPLAY(prev_hour_tens, prev_hour_ones, prev_min_tens, prev_min_ones, prev_sec_tens, prev_sec_ones, prev_hour_tens_dp, prev_hour_ones_dp, prev_min_tens_dp, prev_min_ones_dp, prev_sec_tens_dp, prev_sec_ones_dp);
        } else {
          WRITE_DISPLAY(hour_tens, hour_ones, min_tens, min_ones, sec_tens, sec_ones, hour_tens_dp, hour_ones_dp, min_tens_dp, min_ones_dp, sec_tens_dp, sec_ones_dp);
        }            
      } else {
        WRITE_DISPLAY(hour_tens, hour_ones, min_tens, min_ones, sec_tens, sec_ones, hour_tens_dp, hour_ones_dp, min_tens_dp, min_ones_dp, sec_tens_dp, sec_ones_dp);
      }
      TIMERBLINK++;
    } else {
      WRITE_DISPLAY(8,8,8,8,8,8,1,1,1,1,1,1);
      TIMERBLINK=0;
    }
  } else {    // **NOT** KEEPING THE UNLIT SEGMENTS WARM
    if ( SETTING_CROSSFADE == 1 ) { 
      if ( SECONDS_FRACTION_COUNTER_remain < ((SECONDS_FRACTION_COUNTER_divisor) - SECONDS_FRACTION_COUNTER_divide) ) {
        WRITE_DISPLAY(prev_hour_tens, prev_hour_ones, prev_min_tens, prev_min_ones, prev_sec_tens, prev_sec_ones, prev_hour_tens_dp, prev_hour_ones_dp, prev_min_tens_dp, prev_min_ones_dp, prev_sec_tens_dp, prev_sec_ones_dp);
      } else {
        WRITE_DISPLAY(hour_tens, hour_ones, min_tens, min_ones, sec_tens, sec_ones, hour_tens_dp, hour_ones_dp, min_tens_dp, min_ones_dp, sec_tens_dp, sec_ones_dp);
      }
    } else {
      WRITE_DISPLAY(hour_tens, hour_ones, min_tens, min_ones, sec_tens, sec_ones, hour_tens_dp, hour_ones_dp, min_tens_dp, min_ones_dp, sec_tens_dp, sec_ones_dp);
    }
  }
#endif

#ifdef Z57XM6DV2
      if ( SETTING_CROSSFADE == 1 ) {
        if ( SECONDS_FRACTION_COUNTER_remain < ((SECONDS_FRACTION_COUNTER_divisor) - SECONDS_FRACTION_COUNTER_divide) ) {
          WRITE_DISPLAY(prev_hour_tens, prev_hour_ones, prev_min_tens, prev_min_ones, prev_sec_tens, prev_sec_ones, prev_hour_tens_dp, prev_hour_ones_dp, prev_min_tens_dp, prev_min_ones_dp, prev_sec_tens_dp, prev_sec_ones_dp);
        } else {
          WRITE_DISPLAY(hour_tens, hour_ones, min_tens, min_ones, sec_tens, sec_ones, hour_tens_dp, hour_ones_dp, min_tens_dp, min_ones_dp, sec_tens_dp, sec_ones_dp);
        }            
      } else {
        WRITE_DISPLAY(hour_tens, hour_ones, min_tens, min_ones, sec_tens, sec_ones, hour_tens_dp, hour_ones_dp, min_tens_dp, min_ones_dp, sec_tens_dp, sec_ones_dp);
        //WRITE_DISPLAY(DIGIT_BLANK, DIGIT_BLANK, DIGIT_BLANK, DIGIT_BLANK, DIGIT_BLANK, DIGIT_BLANK, hour_tens_dp, hour_ones_dp, min_tens_dp, min_ones_dp, sec_tens_dp, sec_ones_dp);
      }
#endif
  
}

void time_variables_populate() {
    if ( DISPLAY_OTHER_THEN_TIME == 0 ) {
    sec_ones = timeinfo.tm_sec % 10;  
    sec_tens = timeinfo.tm_sec / 10;
    min_ones = timeinfo.tm_min % 10;
    min_tens = timeinfo.tm_min / 10;
    if ( SETTING_12H == 1 ) {
      if ( timeinfo.tm_hour == 0 ) {  // MIDNIGHT, AM
        hour_ones=2;
        hour_tens=1;
      } else if ( timeinfo.tm_hour >= 1 && timeinfo.tm_hour <= 11 ) { // 1 to 11 AM
        hour_ones = timeinfo.tm_hour % 10;
        hour_tens = timeinfo.tm_hour / 10;
      } else if ( timeinfo.tm_hour == 12 ) {  // NOON, PM
        hour_ones=2;
        hour_tens=1;
      } else {    // THIS IS ANYTHING AFTER 1300
        hour_ones = ((timeinfo.tm_hour)-12) % 10;
        hour_tens = ((timeinfo.tm_hour)-12) / 10; 
      }
    } else {
      hour_ones = timeinfo.tm_hour % 10;
      hour_tens = timeinfo.tm_hour / 10;
    }
    
    if ( SETTING_LZBLANK == 1 ) {
      if ( hour_tens == 0 ) {
        hour_tens=DIGIT_BLANK;
      }
    }
  
    if ( sec_ones % 2 != 0 && SETTING_COLON == 1 ) {    // DECIMAL POINT ON 
      hour_tens_dp = 0; hour_ones_dp = 1; min_tens_dp = 0; min_ones_dp = 1; sec_tens_dp = 0; sec_ones_dp = 0;
    } else {
      hour_tens_dp = 0; hour_ones_dp = 0; min_tens_dp = 0; min_ones_dp = 0; sec_tens_dp = 0; sec_ones_dp = 0;
    }
  }
}

void mqtt_process() {
  if ( SETTING_MQTTENABLE == 1 ) {  // Check if MQTT is enabled
    if ( timeinfo.tm_sec == 50 ) {  // check if connected at 50 seconds
      if (!mqttClient.connected()) {
        mqtt_reconnect();           // If not, reconnect
      }
    }
    if ( timeinfo.tm_sec == 0 ) {
      if (mqttClient.connected()) {
        if ( SETTING_MQTTTEMP_F == 1 ) { mqttClient.publish(mqtt_topic_temp_f, dtostrf(TEMPERATURE_F, 1, 2, mqtt_string_buffer)); }
        if ( SETTING_MQTTTEMP_C == 1 ) { mqttClient.publish(mqtt_topic_temp_c, dtostrf(TEMPERATURE_C, 1, 2, mqtt_string_buffer)); }
        if ( SETTING_MQTTHUM == 1 ) { mqttClient.publish(mqtt_topic_hum, dtostrf(HUMIDITY, 1, 2, mqtt_string_buffer)); }
        if ( SETTING_MQTTLUX == 1 ) { mqttClient.publish(mqtt_topic_lux, dtostrf(lux, 1, 1, mqtt_string_buffer)); }
        if ( SETTING_MQTTbmp_PA == 1 ) { mqttClient.publish(mqtt_topic_bmp_PA, dtostrf(bmp_PA, 1, 2, mqtt_string_buffer)); }
        if ( SETTING_MQTTbmp_HG == 1 ) { mqttClient.publish(mqtt_topic_bmp_HG, dtostrf(bmp_HG, 1, 2, mqtt_string_buffer)); }
        if ( SETTING_MQTTbmp_METERS == 1 ) { mqttClient.publish(mqtt_topic_bmp_METERS, dtostrf(bmp_METERS, 1, 2, mqtt_string_buffer)); }
        if ( SETTING_MQTTbmp_FEET == 1 ) { mqttClient.publish(mqtt_topic_bmp_FEET, dtostrf(bmp_FEET, 1, 1, mqtt_string_buffer)); }
        if ( SETTING_MQTTbmp_TEMP_C == 1 ) { mqttClient.publish(mqtt_topic_bmp_TEMPERATURE_C, dtostrf(bmp_TEMPERATURE_C, 1, 1, mqtt_string_buffer)); }
        if ( SETTING_MQTTRSSI == 1 ) { mqttClient.publish(mqtt_topic_RSSI, ltoa( WiFi.RSSI(), mqtt_string_buffer, 10 ) ); }
      }
    }
  }
  mqttClient.loop();
}

void alt_display_process() {
  if ( SETTING_ALTDISPLAY == 2 ) {  // ALTIMETER
    DISPLAY_OTHER_THEN_TIME=1;
    sec_ones = int(bmp_FEET*10)%10; sec_tens = int(bmp_FEET)%10; min_ones = int(bmp_FEET)%100/10; min_tens = int(bmp_FEET)%1000/100; hour_ones = int(bmp_FEET)%10000/1000; hour_tens = int(bmp_FEET)/10000;
    hour_tens_dp = 0; hour_ones_dp = 0; min_tens_dp = 0; min_ones_dp = 0; sec_tens_dp = 1; sec_ones_dp = 0;
  } else if ( SETTING_ALTDISPLAY == 3 ) { // RSSI
    DISPLAY_OTHER_THEN_TIME=1;
    hour_tens = DIGIT_BLANK; hour_ones = DIGIT_BLANK; min_tens = DIGIT_BLANK; min_ones = DIGIT_NEGATIVE; sec_tens = (abs(WiFi.RSSI()))/10; sec_ones = int((abs(WiFi.RSSI())))%10;
    hour_tens_dp = 0; hour_ones_dp = 0; min_tens_dp = 0; min_ones_dp = 0; sec_tens_dp = 0; sec_ones_dp = 0;
  } else if ( SETTING_ALTDISPLAY == 4 ) { // LUX
    DISPLAY_OTHER_THEN_TIME=1;
    //sec_ones = int(lux)%10; sec_tens = int(lux)%100/10; min_ones = int(lux)%1000/100; min_tens = int(lux)%10000/1000; hour_ones = int(lux)/10000; hour_tens = 10;   // DISPLAY LUX LEVELS
    hour_tens = DIGIT_BLANK; hour_ones = int(lux)/10000; min_tens = int(lux)%10000/1000; min_ones = int(lux)%1000/100; sec_tens = int(lux)%100/10; sec_ones = int(lux)%10;
    hour_tens_dp = 0; hour_ones_dp = 0; min_tens_dp = 0; min_ones_dp = 0; sec_tens_dp = 0; sec_ones_dp = 0;
  } else if ( SETTING_ALTDISPLAY == 5 ) { // BAROMETER
    DISPLAY_OTHER_THEN_TIME=1;
    snprintf(bmp_HG_STRING_BUFFER, sizeof(bmp_HG_STRING_BUFFER), "%.2f", bmp_HG);   // FOR PROPER ROUNDING OF bmp_HG
    #ifdef IV_9_6D_RR
      hour_tens = int(bmp_HG)%100/10; hour_ones = int(bmp_HG)%10; min_tens = int( atof(bmp_HG_STRING_BUFFER) *10)%10; min_ones = int( atof(bmp_HG_STRING_BUFFER) *100)%10; sec_tens = int('H')-55; sec_ones = int('G')-55;
    #else    // Z57 clock cannot display letters
      hour_tens = int(bmp_HG)%100/10; hour_ones = int(bmp_HG)%10; min_tens = int( atof(bmp_HG_STRING_BUFFER) *10)%10; min_ones = int( atof(bmp_HG_STRING_BUFFER) *100)%10; sec_tens = DIGIT_BLANK; sec_ones = DIGIT_BLANK;
    #endif
    hour_tens_dp = 0; hour_ones_dp = 1; min_tens_dp = 0; min_ones_dp = 0; sec_tens_dp = 1; sec_ones_dp = 1;
  }
}

void brightness_and_lux_process() {
  if ( SETTING_LUXMETER == 2 ) {  
    // READ BL1750 LIGHT METER
    lux = lightMeter.readLightLevel();
    //Serial.print("Light: "); Serial.print(lux); Serial.print(" lx"); Serial.print(" ");
    serial_debug("Light: "); serial_debug(lux); serial_debug(" lx"); serial_debug(" ");

    #ifdef IV_9_6D_RR
    // SET PWM ON OUTPUT PIN FOR DIMMING
    if ( lux < 0 ) {
      // do nothing! something is wrong! Sometimes we get a -1 error reading from the sensor
    } else if ( lux > 50 ) {
      //ledcWrite(0, 0);      // FULL BRIGHTNESS  // (0, 0) for inverted output, (0, 255) for regular
      BRIGHTNESS=SETTING_BRIGHTNESS;
    } else if ( lux > 5 ) {
      //ledcWrite(0, 128);    // HALF BRIGHT      // (0, 150 for inverted output, (0, 100) for regular
      BRIGHTNESS=((SETTING_BRIGHTNESS-SETTING_MINBRIGHTNESS)/2) + SETTING_MINBRIGHTNESS; // SET MEDIUM BRIGHTNESS MIDWAY BETWEEN SETTING_BRIGHTNESS AND DIM BRIGHT
    } else {
      //ledcWrite(0, 205);    // DIM              // (0, 200) for inverted output, (0, 50) for regular
      BRIGHTNESS=SETTING_MINBRIGHTNESS;
    }
    #endif

    #ifdef Z57XM6DV2
    // SET PWM ON OUTPUT PIN FOR DIMMING
    if ( lux < 0 ) {
      // do nothing! something is wrong! Sometimes we get a -1 error reading from the sensor
    } else if ( lux > 50 ) {
      //ledcWrite(0, 0);      // FULL BRIGHTNESS  // (0, 0) for inverted output, (0, 255) for regular
      BRIGHTNESS=SETTING_BRIGHTNESS;
    } else if ( lux > 5 ) {
      //ledcWrite(0, 128);    // HALF BRIGHT      // (0, 150 for inverted output, (0, 100) for regular
      //BRIGHTNESS=128;
      BRIGHTNESS=((SETTING_BRIGHTNESS-SETTING_MINBRIGHTNESS)/2) + SETTING_MINBRIGHTNESS; // SET MEDIUM BRIGHTNESS MIDWAY BETWEEN SETTING_BRIGHTNESS AND DIM BRIGHT
    } else {
      //ledcWrite(0, 205);    // DIM              // (0, 200) for inverted output, (0, 50) for regular
      BRIGHTNESS=SETTING_MINBRIGHTNESS;
    }
    #endif
    
  } else if ( SETTING_LUXMETER == 3 ) {   // JUST READ THE LUXMETER, DON'T ADJUST BRIGHTNESS, MAINLY TO DISPLAY THE LUX ON THE WEB INTERFACE
    lux = lightMeter.readLightLevel();  // READ BL1750 LIGHT METER
    serial_debug("Light: "); serial_debug(lux); serial_debug(" lx"); serial_debug(" ");
    BRIGHTNESS=SETTING_BRIGHTNESS;
  } else {
    BRIGHTNESS=SETTING_BRIGHTNESS;
  }
}

void total_runtime_and_uptime_process() {
  TOTAL_RUNTIME_SECONDS_COUNTER++;
  if ( TOTAL_RUNTIME_SECONDS_COUNTER >= 864 ) {  // 1/100th of a day has passed, increment UPTIME and TOTAL_RUNTIME, save to permanent memory, DON'T UPDATE MEMORY TOO OFTEN due to flash wear!
    TOTAL_RUNTIME_SECONDS_COUNTER=0;
    UPTIME=UPTIME + 0.01;
    TOTAL_RUNTIME=TOTAL_RUNTIME + 0.01;
    preferences.begin("clock-settings", false);
    preferences.putFloat("TOTAL_RUNTIME", TOTAL_RUNTIME);
    preferences.end();
  }
}

void setup() {
  esp_task_wdt_init(300, true);  // ENABLE WATCHDOG
  esp_task_wdt_add(NULL);       // ADD CURRENT THREAD
  
  pinMode(CLOCK_PIN, OUTPUT);
  pinMode(DATA_PIN, OUTPUT);
  pinMode(ALARM_PIN, OUTPUT);
  digitalWrite(ALARM_PIN, 0);

  #ifdef IV_9_6D_RR
    pinMode(RCK_PIN, OUTPUT);
  #endif

  // SET UP PWM ON OUTPUT ENABLE PIN FOR DIMMING
  //ledcSetup(0, 400, 8);   //  ledcSetup(uint8_t channel, double freq, uint8_t resolution_bits);
  ledcSetup(2, 1600, 8);   //  ledcSetup(uint8_t channel, double freq, uint8_t resolution_bits);
  ledcAttachPin(OE_PIN, 2);
  //ledcWrite(0, 0);    // INVERTED (0, 0)   // REGULAR (0, 255)
  ledcWrite(2, 255);    // INVERTED (0, 0)   // REGULAR (0, 255)

  BRIGHTNESS=255;   // ADDED in 0.91v BETA4, without this when the light meter was enabled, but no light meter was installed, the brightness was never set and display would be blank

  // CLEAR OUTPUT BITS OF SHIFT REGISTER
  int var = 0;
  while (var <= 65) {
    WRITE_ONEHIGHBIT();    // NO NEED TO WORRY ABOUT INVERTING HERE, IT'S DONE IN THE FUNCTION BOTH LOWBIT AND HIGHBIT
    var++;
  }

  WRITE_DISPLAY_VAR(8,8,8,8,8,8, 0,0,0,0,0,0);
  //WRITE_DISPLAY_VAR(DIGIT_NEGATIVE,DIGIT_NEGATIVE,DIGIT_NEGATIVE,DIGIT_NEGATIVE,DIGIT_NEGATIVE,DIGIT_NEGATIVE, 0, 0, 0, 0, 0, 0); // STARTING OFF WITH LESS SEGMENTS ON SEEMED TO BE WORSE THEN ALL ON, IV-9 clock

  My_timer = timerBegin(1, 80, true);   // NOTE THAT ESP32-C3 ONLY HAS TWO HARDWARE TIMERS, MOST OTHERS HAVE 4, num=hardware timer number, starts at 0       hw_timer_t * timerBegin(uint8_t num, uint16_t divider, bool countUp);
  timerAttachInterrupt(My_timer, &onTimer, true);
  timerAlarmWrite(My_timer, 625, true); // 625 is 1600 hz, 2500 is 400 hz, 4000 is 250hz, 16666 is 60 hz
  timerAlarmEnable(My_timer); //Just Enable

  while ( ACTUAL_BRIGHTNESS < BRIGHTNESS ) {
    ACTUAL_BRIGHTNESS++;
    ledcWrite(2, abs(ACTUAL_BRIGHTNESS-255));
    delay(5);
  }

  // SETTINGS THAT GET STORED IN FLASH
  preferences.begin("clock-settings", false);

  TOTAL_RUNTIME = preferences.getFloat("TOTAL_RUNTIME", 0);   // Get TOTAL_RUNTIME before processing possible reset of variables below

  pinMode(EEPROM_RESET_PIN, INPUT_PULLUP);           // READ PIN (set as a define, check beginning of file) and if low (ground) reset permanent (eeprom/flash) variables to their default values, except TOTAL_RUNTIME
  if ( digitalRead(EEPROM_RESET_PIN) == 0 ) {
    preferences.clear();
    WRITE_DISPLAY_VAR(0,0,0,0,0,0,0,0,0,0,0,0); delay(500); WRITE_DISPLAY_VAR(DIGIT_BLANK,DIGIT_BLANK,DIGIT_BLANK,DIGIT_BLANK,DIGIT_BLANK,DIGIT_BLANK,0,0,0,0,0,0); delay(500);
    WRITE_DISPLAY_VAR(0,0,0,0,0,0,0,0,0,0,0,0); delay(500); WRITE_DISPLAY_VAR(DIGIT_BLANK,DIGIT_BLANK,DIGIT_BLANK,DIGIT_BLANK,DIGIT_BLANK,DIGIT_BLANK,0,0,0,0,0,0); delay(500);
    WRITE_DISPLAY_VAR(0,0,0,0,0,0,0,0,0,0,0,0); delay(500); WRITE_DISPLAY_VAR(DIGIT_BLANK,DIGIT_BLANK,DIGIT_BLANK,DIGIT_BLANK,DIGIT_BLANK,DIGIT_BLANK,0,0,0,0,0,0); delay(500);
  }
    
  preferences.putFloat("TOTAL_RUNTIME", TOTAL_RUNTIME);   // Put TOTAL_RUNTIME back into permanent storage, as we don't want to reset this value

  #ifdef IV_9_6D_RR
    SETTING_BRIGHTNESS = preferences.getInt("BRIGHTNESS", 165);
  #endif
  #ifdef Z57XM6DV2
    SETTING_BRIGHTNESS = preferences.getInt("BRIGHTNESS", 255);
  #endif
  SETTING_MINBRIGHTNESS = preferences.getInt("MINBRIGHTNESS", 50);
  SETTING_LUXMETER = preferences.getInt("LUXMETER", 1);
  //SETTING_DISPLAYTEMP = preferences.getInt("DISPLAYTEMP", 0);
  SETTING_TEMPDISP_SEC = preferences.getInt("TEMPDISP_SEC", 0);
  SETTING_TEMPDISP_DUR = preferences.getInt("TEMPDISP_DUR", 2);
  
  SETTING_TEMP_IN_C = preferences.getInt("TEMP_IN_C", 0);
  SETTING_TEMP_COMP = preferences.getInt("TEMP_COMP", 0);
  SETTING_HUMIDITY_COMP = preferences.getInt("HUMIDITY_COMP", 0);
  SETTING_12H = preferences.getInt("12H", 0);
  SETTING_LZBLANK = preferences.getInt("LZBLANK", 0);
  SETTING_CROSSFADE = preferences.getInt("CROSSFADE", 0);

  SETTING_DATEFORMAT = preferences.getInt("DATEFORMAT", 1);
  SETTING_DATEDISP_SEC = preferences.getInt("DATEDISP_SEC", 0);
  SETTING_DATEDISP_DUR = preferences.getInt("DATEDISP_DUR", 2);
    
  SETTING_CATHODEPP = preferences.getInt("CATHODEPP", 1);
  SETTING_CATHODEPPSTART = preferences.getInt("CATHODEPPSTART", 3);
  SETTING_CATHODEPPDUR = preferences.getInt("CATHODEPPDUR", 1);
  SETTING_CATHODEPPREV = preferences.getInt("CATHODEPPREV", 10);
  SETTING_CATHODEPPREVD = preferences.getInt("CATHODEPPREVD", 4);
  
  SETTING_TIMEZONE = preferences.getString("TIMEZONE", "PST8PDT,M3.2.0,M11.1.0");

  SETTING_TUBEONBOTTOM = preferences.getInt("TUBEONBOTTOM", 1);

  SETTING_COLON = preferences.getInt("COLON", 1);

  SETTING_BMP = preferences.getInt("BMP", 1);

  SETTING_TEMPHUM = preferences.getInt("TEMPHUM", 1);

  SETTING_ALTDISPLAY = preferences.getInt("ALTDISPLAY", 1);

  SETTING_NTPSERVER = preferences.getString("NTPSERVER", "pool.ntp.org");

  SETTING_MQTTENABLE = preferences.getInt("MQTTENABLE", 0);
  SETTING_MQTTBRIGHT = preferences.getInt("MQTTBRIGHT", 0);
  SETTING_MQTTSERVER = preferences.getString("MQTTSERVER", "");
  SETTING_MQTTPORT = preferences.getString("MQTTPORT", "1883");
  SETTING_MQTTUSERNAME = preferences.getString("MQTTUSERNAME", "");
  SETTING_MQTTPASSWORD = preferences.getString("MQTTPASSWORD", "");
  SETTING_MQTTROOT = preferences.getString("MQTTROOT", "");

  SETTING_MQTTTEMP_F = preferences.getInt("MQTTTEMP_F", 1);
  SETTING_MQTTTEMP_C = preferences.getInt("MQTTTEMP_C", 1);
  SETTING_MQTTHUM = preferences.getInt("MQTTHUM", 1);
  SETTING_MQTTLUX = preferences.getInt("MQTTLUX", 1);
  SETTING_MQTTbmp_PA = preferences.getInt("MQTTbmp_PA", 1);
  SETTING_MQTTbmp_HG = preferences.getInt("MQTTbmp_HG", 1);
  SETTING_MQTTbmp_METERS = preferences.getInt("MQTTbmp_METERS", 1);
  SETTING_MQTTbmp_FEET = preferences.getInt("MQTTbmp_FEET", 1);
  SETTING_MQTTbmp_TEMP_C = preferences.getInt("MQTTbmp_TEMP_C", 1);
  SETTING_MQTTRSSI = preferences.getInt("MQTTRSSI", 1);

  SETTING_ALARM1ENABLE = preferences.getInt("ALARM1ENABLE", 0);
  SETTING_ALARM1SUN = preferences.getInt("ALARM1SUN", 0);
  SETTING_ALARM1MON = preferences.getInt("ALARM1MON", 0);
  SETTING_ALARM1TUE = preferences.getInt("ALARM1TUE", 0);
  SETTING_ALARM1WED = preferences.getInt("ALARM1WED", 0);
  SETTING_ALARM1THU = preferences.getInt("ALARM1THU", 0);
  SETTING_ALARM1FRI = preferences.getInt("ALARM1FRI", 0);
  SETTING_ALARM1SAT = preferences.getInt("ALARM1SAT", 0);
  SETTING_ALARM1MONTH = preferences.getInt("ALARM1MONTH", 13);  // DEFAULT ANY IS 13
  SETTING_ALARM1DAY = preferences.getInt("ALARM1DAY", 32);      // DEFAULT ANY IS 32
  SETTING_ALARM1HOUR = preferences.getInt("ALARM1HOUR", 0);
  SETTING_ALARM1MIN = preferences.getInt("ALARM1MIN", 0);
  SETTING_ALARM1DUR = preferences.getInt("ALARM1DUR", 1);
  SETTING_ALARM1TUNE = preferences.getInt("ALARM1TUNE", 1);

  SETTING_ALARM2ENABLE = preferences.getInt("ALARM2ENABLE", 0);
  SETTING_ALARM2SUN = preferences.getInt("ALARM2SUN", 0);
  SETTING_ALARM2MON = preferences.getInt("ALARM2MON", 0);
  SETTING_ALARM2TUE = preferences.getInt("ALARM2TUE", 0);
  SETTING_ALARM2WED = preferences.getInt("ALARM2WED", 0);
  SETTING_ALARM2THU = preferences.getInt("ALARM2THU", 0);
  SETTING_ALARM2FRI = preferences.getInt("ALARM2FRI", 0);
  SETTING_ALARM2SAT = preferences.getInt("ALARM2SAT", 0);
  SETTING_ALARM2MONTH = preferences.getInt("ALARM2MONTH", 13);  // DEFAULT ANY IS 13
  SETTING_ALARM2DAY = preferences.getInt("ALARM2DAY", 32);      // DEFAULT ANY IS 32
  SETTING_ALARM2HOUR = preferences.getInt("ALARM2HOUR", 0);
  SETTING_ALARM2MIN = preferences.getInt("ALARM2MIN", 0);
  SETTING_ALARM2DUR = preferences.getInt("ALARM2DUR", 1);
  SETTING_ALARM2TUNE = preferences.getInt("ALARM2TUNE", 1);

  SETTING_ALARM3ENABLE = preferences.getInt("ALARM3ENABLE", 0);
  SETTING_ALARM3SUN = preferences.getInt("ALARM3SUN", 0);
  SETTING_ALARM3MON = preferences.getInt("ALARM3MON", 0);
  SETTING_ALARM3TUE = preferences.getInt("ALARM3TUE", 0);
  SETTING_ALARM3WED = preferences.getInt("ALARM3WED", 0);
  SETTING_ALARM3THU = preferences.getInt("ALARM3THU", 0);
  SETTING_ALARM3FRI = preferences.getInt("ALARM3FRI", 0);
  SETTING_ALARM3SAT = preferences.getInt("ALARM3SAT", 0);
  SETTING_ALARM3MONTH = preferences.getInt("ALARM3MONTH", 13);  // DEFAULT ANY IS 13
  SETTING_ALARM3DAY = preferences.getInt("ALARM3DAY", 32);      // DEFAULT ANY IS 32
  SETTING_ALARM3HOUR = preferences.getInt("ALARM3HOUR", 0);
  SETTING_ALARM3MIN = preferences.getInt("ALARM3MIN", 0);
  SETTING_ALARM3DUR = preferences.getInt("ALARM3DUR", 1);
  SETTING_ALARM3TUNE = preferences.getInt("ALARM3TUNE", 1);

  SETTING_ONETIMEPLAY = 1; // THIS IS IN THE ALARMS SETTINGS PAGE, BUT WE DON'T STORE IT IN EEPROM

  SETTING_RTTTL1 = preferences.getString("RTTTL1", "");
  SETTING_RTTTL2 = preferences.getString("RTTTL2", "");
  SETTING_RTTTL3 = preferences.getString("RTTTL3", "");

  SETTING_TB1ENABLE = preferences.getInt("TB1ENABLE", 0);
  SETTING_TB1BRIGHT = preferences.getInt("TB1BRIGHT", 0);
  SETTING_TB1SUN = preferences.getInt("TB1SUN", 0);
  SETTING_TB1MON = preferences.getInt("TB1MON", 0);
  SETTING_TB1TUE = preferences.getInt("TB1TUE", 0);
  SETTING_TB1WED = preferences.getInt("TB1WED", 0);
  SETTING_TB1THU = preferences.getInt("TB1THU", 0);
  SETTING_TB1FRI = preferences.getInt("TB1FRI", 0);
  SETTING_TB1SAT = preferences.getInt("TB1SAT", 0);
  SETTING_TB1FROM = preferences.getInt("TB1FROM", 0);
  SETTING_TB1TO = preferences.getInt("TB1TO", 0);

  SETTING_TB2ENABLE = preferences.getInt("TB2ENABLE", 0);
  SETTING_TB2BRIGHT = preferences.getInt("TB2BRIGHT", 0);
  SETTING_TB2SUN = preferences.getInt("TB2SUN", 0);
  SETTING_TB2MON = preferences.getInt("TB2MON", 0);
  SETTING_TB2TUE = preferences.getInt("TB2TUE", 0);
  SETTING_TB2WED = preferences.getInt("TB2WED", 0);
  SETTING_TB2THU = preferences.getInt("TB2THU", 0);
  SETTING_TB2FRI = preferences.getInt("TB2FRI", 0);
  SETTING_TB2SAT = preferences.getInt("TB2SAT", 0);
  SETTING_TB2FROM = preferences.getInt("TB2FROM", 0);
  SETTING_TB2TO = preferences.getInt("TB2TO", 0);

  SETTING_TB3ENABLE = preferences.getInt("TB3ENABLE", 0);
  SETTING_TB3BRIGHT = preferences.getInt("TB3BRIGHT", 0);
  SETTING_TB3SUN = preferences.getInt("TB3SUN", 0);
  SETTING_TB3MON = preferences.getInt("TB3MON", 0);
  SETTING_TB3TUE = preferences.getInt("TB3TUE", 0);
  SETTING_TB3WED = preferences.getInt("TB3WED", 0);
  SETTING_TB3THU = preferences.getInt("TB3THU", 0);
  SETTING_TB3FRI = preferences.getInt("TB3FRI", 0);
  SETTING_TB3SAT = preferences.getInt("TB3SAT", 0);
  SETTING_TB3FROM = preferences.getInt("TB3FROM", 0);
  SETTING_TB3TO = preferences.getInt("TB3TO", 0);

  SETTING_TB4ENABLE = preferences.getInt("TB4ENABLE", 0);
  SETTING_TB4BRIGHT = preferences.getInt("TB4BRIGHT", 0);
  SETTING_TB4SUN = preferences.getInt("TB4SUN", 0);
  SETTING_TB4MON = preferences.getInt("TB4MON", 0);
  SETTING_TB4TUE = preferences.getInt("TB4TUE", 0);
  SETTING_TB4WED = preferences.getInt("TB4WED", 0);
  SETTING_TB4THU = preferences.getInt("TB4THU", 0);
  SETTING_TB4FRI = preferences.getInt("TB4FRI", 0);
  SETTING_TB4SAT = preferences.getInt("TB4SAT", 0);
  SETTING_TB4FROM = preferences.getInt("TB4FROM", 0);
  SETTING_TB4TO = preferences.getInt("TB4TO", 0);

  SETTING_QUICKBOOT = preferences.getInt("QUICKBOOT", 0);

  SETTING_MQTTENABLE = preferences.getInt("MQTTENABLE", 0);

  SETTING_KEEPWARMRATIO = preferences.getInt("KEEPWARMRATIO", 10);
  
  preferences.end();

  timeClient.setPoolServerName(SETTING_NTPSERVER.c_str());

  if ( SETTING_QUICKBOOT == 0 ) {
    demo_digits();
  }

  //char tmp_hostname[30];    // MOVED TO BE DEFINED AS GLOBAL 
  uint8_t mac[6];
  WiFi.macAddress(mac);
  #ifdef IV_9_6D_RR
    sprintf(tmp_hostname, "iv9clock-%02x%02x", mac[4], mac[5]);   // BETA4 REMOVED mac[0] - mac[4] to shorten hostname
  #endif
  #ifdef Z57XM6DV2
    sprintf(tmp_hostname, "Z57clock-%02x%02x", mac[4], mac[5]);   // BETA4 REMOVED mac[0] - mac[4] to shorten hostname
  #endif

  WiFi.mode(WIFI_MODE_NULL);      // HAD TO ADD THIS FOR WiFi.setHostname to work! 12-26-2022  See bug... https://github.com/espressif/arduino-esp32/issues/6700
  WiFi.setHostname(tmp_hostname); // THIS IS THE HOSTNAME REPORTED TO DHCP (usually the one that shows up in your router 'clients' list
  
  // Initialize Serial Monitor
  Serial.begin(115200);
  serial_debugln();
  Server.on("/", rootPage);
  Server.on("/partitions.html", GET_ESP_PARITIONS);
  //config.hostName = "z57clock";
  config.hostName = "tmp_hostname";

  #ifdef IV_9_6D_RR
    config.apid = "iv9clock";
  #else
    config.apid = "z57clock";
  #endif
  config.psk = "neonixie.com";

  config.auth = AC_AUTH_DIGEST;
  config.authScope = AC_AUTHSCOPE_PARTIAL;    // This configures authentication only on the web based firmware update html page (autoconnect.h)
  #ifdef IV_9_6D_RR
    config.username = "iv9clock";
    config.password = "neonixie.com";
  #endif
  #ifdef Z57XM6DV2
    config.username = "z57clock";
    config.password = "neonixie.com";
  #endif
  config.ota = AC_OTA_BUILTIN;  // Turns on web based firmware update (built into autoconnect.h)

  config.menuItems = config.menuItems | AC_MENUITEM_DELETESSID; // THIS ADDS THE OPTION TO DELETE AN SSID 

  config.autoReconnect = true;    // Without this option, when the wifi router was shut off and turned back on, the clock would not attempt a connection again to the wifi, it would just go into soft AP mode. Adding this fixed it and it now attempts to reconnect when the wifi router is back online. This was clued in from https://github.com/Hieromon/AutoConnect/issues/80
  config.ticker = false;          // This is the "ticker" option, which causes a GPIO port to blink during connection and OTA web updates.. which on our ESP32 was defaulted to GPIO 2, which is our output enable pin on the IV-9 clock. This caused the display to flash during web OTA updates. Setting this to false does not modify the OTA update flashes, but should get rid of the other flashes when autoconnect connects to wifi.
  config.tickerPort = BUZZER_PIN; // See comment above on "ticker".. we can't turn off the rapidly flashing GPIO 2 port by setting ticker to false, but we can change it to another port. The BUZZER_PIN should work well and will give us an audible indication of a firmware upload!
  
  Portal.config(config);
  if (Portal.begin()) {
    serial_debugln("HTTP server:" + WiFi.localIP().toString());    
  }

  // Initialize a NTPClient to get time   // THIS IS THE PRIOR OLDER STUFF, NO LONGER USED REALLY
  timeClient.begin();
  timeClient.setPoolServerName(SETTING_NTPSERVER.c_str());

// ADD PAGE TEST
  delay(1000);
  page_add.load(PAGE_ADD);
  page_clockconfigadvanced.load(PAGE_CLOCKCONFIGADVANCED);
  page_alarm.load(PAGE_ALARM);
  page_timed_brightness.load(PAGE_TIMED_BRIGHTNESS);
  //page_results.load(PAGE_RESULTS);
  //Portal.join({ page_add, page_results });
  Portal.join({ page_add, page_clockconfigadvanced, page_alarm, page_timed_brightness });
  Portal.on("/clockconfig", onclockconfig);
  Portal.on("/clockconfigadvanced", clockconfigadvanced);
  Portal.on("/alarm", alarmfunction);
  Portal.on("/timed_brightness", timed_brightness);
  //Portal.on("/results", onResults);
  Portal.begin();
// END ADD PAGE TEST

  if ( SETTING_QUICKBOOT == 0 ) {
    // DISPLAY LOCAL IP ADDRESS
    //WRITE_DISPLAY(WiFi.localIP()[0]/100, WiFi.localIP()[0]%100/10, WiFi.localIP()[0]%10, WiFi.localIP()[1]/100, WiFi.localIP()[1]%100/10, WiFi.localIP()[1]%10, 0, 0, 1, 0, 0, 0);
    WRITE_DISPLAY_VAR(WiFi.localIP()[0]/100, WiFi.localIP()[0]%100/10, WiFi.localIP()[0]%10, WiFi.localIP()[1]/100, WiFi.localIP()[1]%100/10, WiFi.localIP()[1]%10, 0, 0, 1, 0, 0, 0);
    //hour_tens = WiFi.localIP()[0]/100; hour_ones = WiFi.localIP()[0]%100/10; min_tens = WiFi.localIP()[0]%10; min_ones = WiFi.localIP()[1]/100; sec_tens = WiFi.localIP()[1]%100/10; sec_ones = WiFi.localIP()[1]%10;
    //hour_tens_dp = 0; hour_ones_dp = 0; min_tens_dp = 1; min_ones_dp = 0; sec_tens_dp = 0; sec_ones_dp = 0;
    delay(4000);
    //WRITE_DISPLAY(WiFi.localIP()[2]/100, WiFi.localIP()[2]%100/10, WiFi.localIP()[2]%10, WiFi.localIP()[3]/100, WiFi.localIP()[3]%100/10, WiFi.localIP()[3]%10, 0, 0, 1, 0, 0, 0);
    WRITE_DISPLAY_VAR(WiFi.localIP()[2]/100, WiFi.localIP()[2]%100/10, WiFi.localIP()[2]%10, WiFi.localIP()[3]/100, WiFi.localIP()[3]%100/10, WiFi.localIP()[3]%10, 0, 0, 1, 0, 0, 0);
    //hour_tens = WiFi.localIP()[2]/100; hour_ones = WiFi.localIP()[2]%100/10; min_tens = WiFi.localIP()[2]%10; min_ones = WiFi.localIP()[3]/100; sec_tens = WiFi.localIP()[3]%100/10; sec_ones = WiFi.localIP()[3]%10;
    delay(4000);        
  }

  initTime(SETTING_TIMEZONE);   // PST8PDT,M3.2.0,M11.1.0   // THIS IS THE ACTUAL ONE THAT WORKS WITH TIMEZONES NOW 06/04/2022
  printLocalTime();

  // OTA STUFF START

  // Port defaults to 3232
  // ArduinoOTA.setPort(3232);

  // Hostname defaults to esp3232-[MAC]
  // ArduinoOTA.setHostname("myesp32");

  //char tmp_hostname[30];    // MOVED ABOVE
  //uint8_t mac[6];
  //WiFi.macAddress(mac);
  //sprintf(tmp_hostname, "Z57clock-%02x%02x", mac[4], mac[5]);   // BETA4 REMOVED mac[0] - mac[4] to shorten hostname
  
  ArduinoOTA.setHostname(tmp_hostname);

  // No authentication by default
  // ArduinoOTA.setPassword("admin");
  ArduinoOTA.setPassword("neonixie.com");

  // Password can be set with it's md5 value as well
  // MD5(admin) = 21232f297a57a5a743894a0e4a801fc3
  // ArduinoOTA.setPasswordHash("21232f297a57a5a743894a0e4a801fc3");

        ArduinoOTA
    .onStart([]() {
      String type;
      if (ArduinoOTA.getCommand() == U_FLASH)
        type = "sketch";
      else // U_SPIFFS
        type = "filesystem";

      // NOTE: if updating SPIFFS this would be the place to unmount SPIFFS using SPIFFS.end()
      serial_debugln("Start updating " + type);
    })
    .onEnd([]() {
      serial_debugln("\nEnd");
    })
    .onProgress([](unsigned int progress, unsigned int total) {
      serial_debug_printf("Progress: %u%%\r", (progress / (total / 100)));
    })
    .onError([](ota_error_t error) {
      serial_debug_printf("Error[%u]: ", error);
      if (error == OTA_AUTH_ERROR) serial_debugln("Auth Failed");
      else if (error == OTA_BEGIN_ERROR) serial_debugln("Begin Failed");
      else if (error == OTA_CONNECT_ERROR) serial_debugln("Connect Failed");
      else if (error == OTA_RECEIVE_ERROR) serial_debugln("Receive Failed");
      else if (error == OTA_END_ERROR) serial_debugln("End Failed");
    });

  ArduinoOTA.begin();
  // OTA STUFF END

  Wire.begin();
  lightMeter.begin();

  // START AHT20 TEMP HUMIDITY SENSOR
  aht.begin();

  #ifdef BMP_ENABLE
  // BMP280 sensor
  bmp.begin(0x77);
  /* Default settings from datasheet. */
  bmp.setSampling(Adafruit_BMP280::MODE_NORMAL,     /* Operating Mode. */
    Adafruit_BMP280::SAMPLING_X2,     /* Temp. oversampling */
    Adafruit_BMP280::SAMPLING_X16,    /* Pressure oversampling */
    Adafruit_BMP280::FILTER_X16,      /* Filtering. */
    Adafruit_BMP280::STANDBY_MS_500); /* Standby time. */
  #endif

  char mqtt_temp_root[60];
  if ( SETTING_MQTTROOT != "" ) {
    snprintf(mqtt_temp_root, sizeof(mqtt_temp_root), "%s", SETTING_MQTTROOT.c_str());
  } else {
    snprintf(mqtt_temp_root, sizeof(mqtt_temp_root), "clock/neonixie/%s/", tmp_hostname);
  }

  snprintf(mqtt_topic_temp_f, sizeof(mqtt_topic_temp_f), "%stemperature_f", mqtt_temp_root);
  snprintf(mqtt_topic_temp_c, sizeof(mqtt_topic_temp_c), "%stemperature_c", mqtt_temp_root);
  snprintf(mqtt_topic_hum, sizeof(mqtt_topic_hum), "%shumidity", mqtt_temp_root);
  snprintf(mqtt_topic_lux, sizeof(mqtt_topic_lux), "%slux", mqtt_temp_root);  
  snprintf(mqtt_topic_bmp_PA, sizeof(mqtt_topic_bmp_PA), "%sbmp_PA", mqtt_temp_root);  
  snprintf(mqtt_topic_bmp_HG, sizeof(mqtt_topic_bmp_HG), "%sbmp_HG", mqtt_temp_root);  
  snprintf(mqtt_topic_bmp_METERS, sizeof(mqtt_topic_bmp_METERS), "%sbmp_METERS", mqtt_temp_root);  
  snprintf(mqtt_topic_bmp_FEET, sizeof(mqtt_topic_bmp_FEET), "%sbmp_FEET", mqtt_temp_root);  
  snprintf(mqtt_topic_bmp_TEMPERATURE_C, sizeof(mqtt_topic_bmp_TEMPERATURE_C), "%sbmp_TEMPERATURE_C", mqtt_temp_root);  
  snprintf(mqtt_topic_RSSI, sizeof(mqtt_topic_RSSI), "%sRSSI", mqtt_temp_root);  

  snprintf(mqtt_topic_subscribe_brightness, sizeof(mqtt_topic_subscribe_brightness), "%ssubscribe_brightness", mqtt_temp_root);  

  mqtt_reconnect();

//  // ORIGINAL BEEP CODE, REMOVED FOR TESTING MUSICAL STUFF
//  pinMode(BUZZER_PIN, OUTPUT);
//  // do we need this for the buzzer ? //ledcSetup(4, 1600, 8);   //  ledcSetup(uint8_t channel, double freq, uint8_t resolution_bits);  // ORIGINAL CHANNEL 0, TESTING OUT OTHERS 03/19/2023
//  ledcAttachPin(BUZZER_PIN, 2); // void ledcAttachPin(uint8_t pin, uint8_t chan);
//  //ledcWrite(2, 10); // void ledcWrite(uint8_t chan, uint32_t duty);
//  //ledcWrite(0, 0);    // INVERTED (0, 0)   // REGULAR (0, 255)
//
//  ledcWriteNote(2, NOTE_E, 4);  // SEEMS THAT IF WE DON'T PLAY A NOTE AND THEN SILENCE IT, THE BUZZER WILL OMIT A LOW VOLUME TONE UNTIL WE DO, SO WE DO THIS AT BOOTUP
//  delay(500);
//  ledcWriteNote(2, NOTE_C, 4);   
//  delay(500);
//  ledcWriteTone(2, 0);

}

// WIFI RECONNECT TEST // ADDED 01/10/2022, to see if it fixes reconnect problem. // 07/11/2022 SEEMS TO RECONNECT WHEN NEEDED
unsigned long previousMillis = 0;
unsigned long interval = 30000;

unsigned long NTP_previousMillis = 9999999;   // MUST BE LARGER THEN interval
unsigned long NTP_interval = 300000;   // INTERVAL IN MILLISECONDS TO RUN THE timeClient.update() FUNCTION.  60000 = 1 minute  300000 = 5 minutes

unsigned long NOTES_previousMillis = 9999999;   // MUST BE LARGER THEN interval
unsigned long NOTES_interval = 500;   // INTERVAL IN MILLISECONDS TO RUN THE timeClient.update() FUNCTION.  60000 = 1 minute  300000 = 5 minutes

unsigned long SECONDS_FRACTION_COUNTER_currentMillis, SECONDS_FRACTION_COUNTER_previousMillis;
unsigned long ACTUAL_BRIGHTNESS_currentMillis, ACTUAL_BRIGHTNESS_previousMillis;

void loop() {
  // NEW STUFF WITH TIMEZONE
  //printLocalTime();
  if(!getLocalTime(&timeinfo)){       //struct tm timeinfo;   // MOVED TO GLOBAL
    serial_debugln("Failed to obtain time getLocalTime");
    //return;     // REMOVED FOR TEST 03-11-2023
  }
  
  ArduinoOTA.handle();    // OTA
  Portal.handleClient();
  
  unsigned long NTP_currentMillis = millis();
  if ( NTP_currentMillis - NTP_previousMillis >=NTP_interval ) {
    serial_debug("timeClient.update start");
    timeClient.update();      // NOTE, POSSIBLY DUE TO DNS DELAYS OR ?? THIS FUNCTION WILL SOMETIMES DELAY BY SEVERAL SECONDS. SET TO RUN ONLY PERIODICALLY SO AS NOT TO DISTRUPT THE CLOCK DISPLAY
    serial_debug("timeClient.update end");
    NTP_previousMillis = NTP_currentMillis;
  }

// WIFI RECONNECT TEST // ADDED 01/10/2022, to see if it fixes reconnect problem. // 05/29/2022 SEEMS TO WORK WELL
unsigned long currentMillis = millis();
// if WiFi is down, try reconnecting
if ((WiFi.status() != WL_CONNECTED) && (currentMillis - previousMillis >=interval)) {
  serial_debug(millis());
  serial_debugln("Reconnecting to WiFi...");
  WiFi.disconnect();
  WiFi.reconnect();
  previousMillis = currentMillis;
}

// START ONCE PER SECOND LOOP
if ( previous_tm_sec != timeinfo.tm_sec ) {
  previous_tm_sec=timeinfo.tm_sec;
  DISPLAY_OTHER_THEN_TIME=0;
  SECONDS_FRACTION_COUNTER=0;
  
  prev_sec_ones = sec_ones; prev_sec_tens = sec_tens; prev_min_ones = min_ones; prev_min_tens = min_tens; prev_hour_ones = hour_ones; prev_hour_tens = hour_tens;
  prev_sec_ones_dp = sec_ones_dp; prev_sec_tens_dp = sec_tens_dp; prev_min_ones_dp = min_ones_dp; prev_min_tens_dp = min_tens_dp; prev_hour_ones_dp = hour_ones_dp; prev_hour_tens_dp = hour_tens_dp;

  esp_task_wdt_reset();   // Watchdog timer reset
  total_runtime_and_uptime_process();
  brightness_and_lux_process();

  #ifdef BMP_ENABLE
    read_bmp(); // READ BMP280 sensor, we currently just display it on the main webpage
  #endif
  display_temperature();
  display_date();

  #ifdef Z57XM6DV2
    if ( SETTING_CATHODEPP != 1 ) { cathodepp(); }
  #endif

  timed_brightness_process();
  alt_display_process();
  mqtt_process();
  alarm_process();
  time_variables_populate();

  strftime(TIME_STRING, sizeof(TIME_STRING), "%A, %B %d %Y %H:%M:%S %F %Z %z", &timeinfo);    //strftime(TIME_STRING, sizeof(TIME_STRING), "%A, %B %d %Y %H:%M:%S %F (%Z) weekday(sun): %u weeknumber(sun): %U weekday(mon): %w weeknumber(mon): %W TimeZoneOffset: %z ", &timeinfo);
  serial_debug_x2(&timeinfo, "%A, %B %d %Y %H:%M:%S zone %Z %z ");   // ERROR, 2 arguments
  serial_debugln("");   // PUT A FINAL RETURN CHARACTER OUT THE SERIAL PORT
} 
// END ONCE PER SECOND LOOP

// DO NOT PUT DELAYS IN THIS MAIN LOOP OR THINGS LIKE CROSS FADING WILL NOT WORK CORRECTLY

  ACTUAL_BRIGHTNESS_currentMillis = millis();
  if ( ACTUAL_BRIGHTNESS < BRIGHTNESS ) {
    if ( ACTUAL_BRIGHTNESS_currentMillis - ACTUAL_BRIGHTNESS_previousMillis >= 5 ) {
      ACTUAL_BRIGHTNESS++;
      ACTUAL_BRIGHTNESS_previousMillis = ACTUAL_BRIGHTNESS_currentMillis;
    }
  } else if ( ACTUAL_BRIGHTNESS > BRIGHTNESS ) {
    if ( ACTUAL_BRIGHTNESS_currentMillis - ACTUAL_BRIGHTNESS_previousMillis >= 5 ) {
      ACTUAL_BRIGHTNESS--;
      ACTUAL_BRIGHTNESS_previousMillis = ACTUAL_BRIGHTNESS_currentMillis;
    }
  }
  ledcWrite(2, abs(ACTUAL_BRIGHTNESS-255));

  // THIS IS A MILLISECOND COUNTER FOR DIGIT CROSS FADING
  SECONDS_FRACTION_COUNTER_currentMillis = millis();
  if ( SECONDS_FRACTION_COUNTER_currentMillis - SECONDS_FRACTION_COUNTER_previousMillis >= 1 ) { // this should trigger every 1 ms
    SECONDS_FRACTION_COUNTER++;
    SECONDS_FRACTION_COUNTER_previousMillis = SECONDS_FRACTION_COUNTER_currentMillis;
  }

  make_noise_check(); // CHECK IF WE NEED TO MAKE NOISE
    
}
